# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.documents_single_get_response_data_relationships_attachments import (
        DocumentsSingleGetResponseDataRelationshipsAttachments,
    )
    from ..models.documents_single_get_response_data_relationships_author import (
        DocumentsSingleGetResponseDataRelationshipsAuthor,
    )
    from ..models.documents_single_get_response_data_relationships_branched_from import (
        DocumentsSingleGetResponseDataRelationshipsBranchedFrom,
    )
    from ..models.documents_single_get_response_data_relationships_comments import (
        DocumentsSingleGetResponseDataRelationshipsComments,
    )
    from ..models.documents_single_get_response_data_relationships_derived_from import (
        DocumentsSingleGetResponseDataRelationshipsDerivedFrom,
    )
    from ..models.documents_single_get_response_data_relationships_project import (
        DocumentsSingleGetResponseDataRelationshipsProject,
    )
    from ..models.documents_single_get_response_data_relationships_updated_by import (
        DocumentsSingleGetResponseDataRelationshipsUpdatedBy,
    )
    from ..models.documents_single_get_response_data_relationships_variant import (
        DocumentsSingleGetResponseDataRelationshipsVariant,
    )


T = TypeVar("T", bound="DocumentsSingleGetResponseDataRelationships")


@_attrs_define
class DocumentsSingleGetResponseDataRelationships:
    """
    Attributes:
        attachments (Union[Unset, DocumentsSingleGetResponseDataRelationshipsAttachments]):
        author (Union[Unset, DocumentsSingleGetResponseDataRelationshipsAuthor]):
        branched_from (Union[Unset, DocumentsSingleGetResponseDataRelationshipsBranchedFrom]):
        comments (Union[Unset, DocumentsSingleGetResponseDataRelationshipsComments]):
        derived_from (Union[Unset, DocumentsSingleGetResponseDataRelationshipsDerivedFrom]):
        project (Union[Unset, DocumentsSingleGetResponseDataRelationshipsProject]):
        updated_by (Union[Unset, DocumentsSingleGetResponseDataRelationshipsUpdatedBy]):
        variant (Union[Unset, DocumentsSingleGetResponseDataRelationshipsVariant]):
    """

    attachments: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsAttachments"
    ] = UNSET
    author: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsAuthor"
    ] = UNSET
    branched_from: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsBranchedFrom"
    ] = UNSET
    comments: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsComments"
    ] = UNSET
    derived_from: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsDerivedFrom"
    ] = UNSET
    project: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsProject"
    ] = UNSET
    updated_by: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsUpdatedBy"
    ] = UNSET
    variant: Union[
        Unset, "DocumentsSingleGetResponseDataRelationshipsVariant"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        attachments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = self.attachments.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        branched_from: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.branched_from, Unset):
            branched_from = self.branched_from.to_dict()

        comments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = self.comments.to_dict()

        derived_from: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.derived_from, Unset):
            derived_from = self.derived_from.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        updated_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        variant: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.variant, Unset):
            variant = self.variant.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if author is not UNSET:
            field_dict["author"] = author
        if branched_from is not UNSET:
            field_dict["branchedFrom"] = branched_from
        if comments is not UNSET:
            field_dict["comments"] = comments
        if derived_from is not UNSET:
            field_dict["derivedFrom"] = derived_from
        if project is not UNSET:
            field_dict["project"] = project
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if variant is not UNSET:
            field_dict["variant"] = variant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.documents_single_get_response_data_relationships_attachments import (
            DocumentsSingleGetResponseDataRelationshipsAttachments,
        )
        from ..models.documents_single_get_response_data_relationships_author import (
            DocumentsSingleGetResponseDataRelationshipsAuthor,
        )
        from ..models.documents_single_get_response_data_relationships_branched_from import (
            DocumentsSingleGetResponseDataRelationshipsBranchedFrom,
        )
        from ..models.documents_single_get_response_data_relationships_comments import (
            DocumentsSingleGetResponseDataRelationshipsComments,
        )
        from ..models.documents_single_get_response_data_relationships_derived_from import (
            DocumentsSingleGetResponseDataRelationshipsDerivedFrom,
        )
        from ..models.documents_single_get_response_data_relationships_project import (
            DocumentsSingleGetResponseDataRelationshipsProject,
        )
        from ..models.documents_single_get_response_data_relationships_updated_by import (
            DocumentsSingleGetResponseDataRelationshipsUpdatedBy,
        )
        from ..models.documents_single_get_response_data_relationships_variant import (
            DocumentsSingleGetResponseDataRelationshipsVariant,
        )

        d = src_dict.copy()
        _attachments = d.pop("attachments", UNSET)
        attachments: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsAttachments
        ]
        if isinstance(_attachments, Unset):
            attachments = UNSET
        else:
            attachments = DocumentsSingleGetResponseDataRelationshipsAttachments.from_dict(
                _attachments
            )

        _author = d.pop("author", UNSET)
        author: Union[Unset, DocumentsSingleGetResponseDataRelationshipsAuthor]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = (
                DocumentsSingleGetResponseDataRelationshipsAuthor.from_dict(
                    _author
                )
            )

        _branched_from = d.pop("branchedFrom", UNSET)
        branched_from: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsBranchedFrom
        ]
        if isinstance(_branched_from, Unset):
            branched_from = UNSET
        else:
            branched_from = DocumentsSingleGetResponseDataRelationshipsBranchedFrom.from_dict(
                _branched_from
            )

        _comments = d.pop("comments", UNSET)
        comments: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsComments
        ]
        if isinstance(_comments, Unset):
            comments = UNSET
        else:
            comments = (
                DocumentsSingleGetResponseDataRelationshipsComments.from_dict(
                    _comments
                )
            )

        _derived_from = d.pop("derivedFrom", UNSET)
        derived_from: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsDerivedFrom
        ]
        if isinstance(_derived_from, Unset):
            derived_from = UNSET
        else:
            derived_from = DocumentsSingleGetResponseDataRelationshipsDerivedFrom.from_dict(
                _derived_from
            )

        _project = d.pop("project", UNSET)
        project: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsProject
        ]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = (
                DocumentsSingleGetResponseDataRelationshipsProject.from_dict(
                    _project
                )
            )

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsUpdatedBy
        ]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = (
                DocumentsSingleGetResponseDataRelationshipsUpdatedBy.from_dict(
                    _updated_by
                )
            )

        _variant = d.pop("variant", UNSET)
        variant: Union[
            Unset, DocumentsSingleGetResponseDataRelationshipsVariant
        ]
        if isinstance(_variant, Unset):
            variant = UNSET
        else:
            variant = (
                DocumentsSingleGetResponseDataRelationshipsVariant.from_dict(
                    _variant
                )
            )

        documents_single_get_response_data_relationships_obj = cls(
            attachments=attachments,
            author=author,
            branched_from=branched_from,
            comments=comments,
            derived_from=derived_from,
            project=project,
            updated_by=updated_by,
            variant=variant,
        )

        documents_single_get_response_data_relationships_obj.additional_properties = (
            d
        )
        return documents_single_get_response_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Polarion API module serving all data classes, clients and errors."""
from importlib import metadata

try:
    __version__ = metadata.version("polarion_rest_api_client")
except metadata.PackageNotFoundError:  # pragma: no cover
    __version__ = "0.0.0+unknown"
del metadata

from polarion_rest_api_client.client import PolarionClient
from polarion_rest_api_client.clients.projects import ProjectClient
from polarion_rest_api_client.data_models import *
from polarion_rest_api_client.errors import *



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.documents_list_post_request_data_item_attributes_home_page_content import (
        DocumentsListPostRequestDataItemAttributesHomePageContent,
    )
    from ..models.documents_list_post_request_data_item_attributes_outline_numbering import (
        DocumentsListPostRequestDataItemAttributesOutlineNumbering,
    )
    from ..models.documents_list_post_request_data_item_attributes_rendering_layouts_item import (
        DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem,
    )


T = TypeVar("T", bound="DocumentsListPostRequestDataItemAttributes")


@_attrs_define
class DocumentsListPostRequestDataItemAttributes:
    """
    Attributes:
        auto_suspect (Union[Unset, bool]):
        home_page_content (Union[Unset, DocumentsListPostRequestDataItemAttributesHomePageContent]):
        module_name (Union[Unset, str]):  Example: MyDocumentId.
        outline_numbering (Union[Unset, DocumentsListPostRequestDataItemAttributesOutlineNumbering]):
        rendering_layouts (Union[Unset, List['DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem']]):
        status (Union[Unset, str]):  Example: draft.
        structure_link_role (Union[Unset, str]):  Example: relates_to.
        title (Union[Unset, str]):  Example: Title.
        type (Union[Unset, str]):  Example: req_specification.
        uses_outline_numbering (Union[Unset, bool]):
    """

    auto_suspect: Union[Unset, bool] = UNSET
    home_page_content: Union[
        Unset, "DocumentsListPostRequestDataItemAttributesHomePageContent"
    ] = UNSET
    module_name: Union[Unset, str] = UNSET
    outline_numbering: Union[
        Unset, "DocumentsListPostRequestDataItemAttributesOutlineNumbering"
    ] = UNSET
    rendering_layouts: Union[
        Unset,
        List["DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem"],
    ] = UNSET
    status: Union[Unset, str] = UNSET
    structure_link_role: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    uses_outline_numbering: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        auto_suspect = self.auto_suspect

        home_page_content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.home_page_content, Unset):
            home_page_content = self.home_page_content.to_dict()

        module_name = self.module_name

        outline_numbering: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.outline_numbering, Unset):
            outline_numbering = self.outline_numbering.to_dict()

        rendering_layouts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rendering_layouts, Unset):
            rendering_layouts = []
            for rendering_layouts_item_data in self.rendering_layouts:
                rendering_layouts_item = rendering_layouts_item_data.to_dict()
                rendering_layouts.append(rendering_layouts_item)

        status = self.status

        structure_link_role = self.structure_link_role

        title = self.title

        type = self.type

        uses_outline_numbering = self.uses_outline_numbering

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if auto_suspect is not UNSET:
            field_dict["autoSuspect"] = auto_suspect
        if home_page_content is not UNSET:
            field_dict["homePageContent"] = home_page_content
        if module_name is not UNSET:
            field_dict["moduleName"] = module_name
        if outline_numbering is not UNSET:
            field_dict["outlineNumbering"] = outline_numbering
        if rendering_layouts is not UNSET:
            field_dict["renderingLayouts"] = rendering_layouts
        if status is not UNSET:
            field_dict["status"] = status
        if structure_link_role is not UNSET:
            field_dict["structureLinkRole"] = structure_link_role
        if title is not UNSET:
            field_dict["title"] = title
        if type is not UNSET:
            field_dict["type"] = type
        if uses_outline_numbering is not UNSET:
            field_dict["usesOutlineNumbering"] = uses_outline_numbering

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.documents_list_post_request_data_item_attributes_home_page_content import (
            DocumentsListPostRequestDataItemAttributesHomePageContent,
        )
        from ..models.documents_list_post_request_data_item_attributes_outline_numbering import (
            DocumentsListPostRequestDataItemAttributesOutlineNumbering,
        )
        from ..models.documents_list_post_request_data_item_attributes_rendering_layouts_item import (
            DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem,
        )

        d = src_dict.copy()
        auto_suspect = d.pop("autoSuspect", UNSET)

        _home_page_content = d.pop("homePageContent", UNSET)
        home_page_content: Union[
            Unset, DocumentsListPostRequestDataItemAttributesHomePageContent
        ]
        if isinstance(_home_page_content, Unset):
            home_page_content = UNSET
        else:
            home_page_content = DocumentsListPostRequestDataItemAttributesHomePageContent.from_dict(
                _home_page_content
            )

        module_name = d.pop("moduleName", UNSET)

        _outline_numbering = d.pop("outlineNumbering", UNSET)
        outline_numbering: Union[
            Unset, DocumentsListPostRequestDataItemAttributesOutlineNumbering
        ]
        if isinstance(_outline_numbering, Unset):
            outline_numbering = UNSET
        else:
            outline_numbering = DocumentsListPostRequestDataItemAttributesOutlineNumbering.from_dict(
                _outline_numbering
            )

        rendering_layouts = []
        _rendering_layouts = d.pop("renderingLayouts", UNSET)
        for rendering_layouts_item_data in _rendering_layouts or []:
            rendering_layouts_item = DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem.from_dict(
                rendering_layouts_item_data
            )

            rendering_layouts.append(rendering_layouts_item)

        status = d.pop("status", UNSET)

        structure_link_role = d.pop("structureLinkRole", UNSET)

        title = d.pop("title", UNSET)

        type = d.pop("type", UNSET)

        uses_outline_numbering = d.pop("usesOutlineNumbering", UNSET)

        documents_list_post_request_data_item_attributes_obj = cls(
            auto_suspect=auto_suspect,
            home_page_content=home_page_content,
            module_name=module_name,
            outline_numbering=outline_numbering,
            rendering_layouts=rendering_layouts,
            status=status,
            structure_link_role=structure_link_role,
            title=title,
            type=type,
            uses_outline_numbering=uses_outline_numbering,
        )

        documents_list_post_request_data_item_attributes_obj.additional_properties = (
            d
        )
        return documents_list_post_request_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class PlansSingleGetResponseDataRelationshipsWorkItemsDataItemType(str, Enum):
    WORKITEMS = "workitems"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workrecords_single_get_response_data import (
        WorkrecordsSingleGetResponseData,
    )
    from ..models.workrecords_single_get_response_included_item import (
        WorkrecordsSingleGetResponseIncludedItem,
    )
    from ..models.workrecords_single_get_response_links import (
        WorkrecordsSingleGetResponseLinks,
    )


T = TypeVar("T", bound="WorkrecordsSingleGetResponse")


@_attrs_define
class WorkrecordsSingleGetResponse:
    """
    Attributes:
        data (Union[Unset, WorkrecordsSingleGetResponseData]):
        included (Union[Unset, List['WorkrecordsSingleGetResponseIncludedItem']]): Related entities might be returned,
            see <a href="https://docs.sw.siemens.com/en-
            US/doc/230235217/PL20231017526942799.polarion_help_sc.xid2134849/xid2134871" target="_blank">REST API User
            Guide</a>.
        links (Union[Unset, WorkrecordsSingleGetResponseLinks]):
    """

    data: Union[Unset, "WorkrecordsSingleGetResponseData"] = UNSET
    included: Union[
        Unset, List["WorkrecordsSingleGetResponseIncludedItem"]
    ] = UNSET
    links: Union[Unset, "WorkrecordsSingleGetResponseLinks"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        included: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.included, Unset):
            included = []
            for included_item_data in self.included:
                included_item = included_item_data.to_dict()
                included.append(included_item)

        links: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.links, Unset):
            links = self.links.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data is not UNSET:
            field_dict["data"] = data
        if included is not UNSET:
            field_dict["included"] = included
        if links is not UNSET:
            field_dict["links"] = links

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workrecords_single_get_response_data import (
            WorkrecordsSingleGetResponseData,
        )
        from ..models.workrecords_single_get_response_included_item import (
            WorkrecordsSingleGetResponseIncludedItem,
        )
        from ..models.workrecords_single_get_response_links import (
            WorkrecordsSingleGetResponseLinks,
        )

        d = src_dict.copy()
        _data = d.pop("data", UNSET)
        data: Union[Unset, WorkrecordsSingleGetResponseData]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = WorkrecordsSingleGetResponseData.from_dict(_data)

        included = []
        _included = d.pop("included", UNSET)
        for included_item_data in _included or []:
            included_item = WorkrecordsSingleGetResponseIncludedItem.from_dict(
                included_item_data
            )

            included.append(included_item)

        _links = d.pop("links", UNSET)
        links: Union[Unset, WorkrecordsSingleGetResponseLinks]
        if isinstance(_links, Unset):
            links = UNSET
        else:
            links = WorkrecordsSingleGetResponseLinks.from_dict(_links)

        workrecords_single_get_response_obj = cls(
            data=data,
            included=included,
            links=links,
        )

        workrecords_single_get_response_obj.additional_properties = d
        return workrecords_single_get_response_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PagesSingleGetResponseDataAttributes")


@_attrs_define
class PagesSingleGetResponseDataAttributes:
    """
    Attributes:
        created (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        page_name (Union[Unset, str]):  Example: MyRichPageId.
        space_id (Union[Unset, str]):  Example: MySpaceId.
        title (Union[Unset, str]):  Example: Title.
        updated (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
    """

    created: Union[Unset, datetime.datetime] = UNSET
    page_name: Union[Unset, str] = UNSET
    space_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        page_name = self.page_name

        space_id = self.space_id

        title = self.title

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created is not UNSET:
            field_dict["created"] = created
        if page_name is not UNSET:
            field_dict["pageName"] = page_name
        if space_id is not UNSET:
            field_dict["spaceId"] = space_id
        if title is not UNSET:
            field_dict["title"] = title
        if updated is not UNSET:
            field_dict["updated"] = updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        page_name = d.pop("pageName", UNSET)

        space_id = d.pop("spaceId", UNSET)

        title = d.pop("title", UNSET)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset):
            updated = UNSET
        else:
            updated = isoparse(_updated)

        pages_single_get_response_data_attributes_obj = cls(
            created=created,
            page_name=page_name,
            space_id=space_id,
            title=title,
            updated=updated,
        )

        pages_single_get_response_data_attributes_obj.additional_properties = d
        return pages_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MergeDocumentRequestBody")


@_attrs_define
class MergeDocumentRequestBody:
    """
    Attributes:
        create_baseline (Union[Unset, bool]): Specifies whether the Baseline should be created. Example: True.
        user_filter (Union[Unset, str]): Specifies the query to filter the source Work Items for the merge. Example:
            status:open.
    """

    create_baseline: Union[Unset, bool] = UNSET
    user_filter: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        create_baseline = self.create_baseline

        user_filter = self.user_filter

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if create_baseline is not UNSET:
            field_dict["createBaseline"] = create_baseline
        if user_filter is not UNSET:
            field_dict["userFilter"] = user_filter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        create_baseline = d.pop("createBaseline", UNSET)

        user_filter = d.pop("userFilter", UNSET)

        merge_document_request_body_obj = cls(
            create_baseline=create_baseline,
            user_filter=user_filter,
        )

        merge_document_request_body_obj.additional_properties = d
        return merge_document_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.jobs_single_post_response_data_attributes_status import (
        JobsSinglePostResponseDataAttributesStatus,
    )


T = TypeVar("T", bound="JobsSinglePostResponseDataAttributes")


@_attrs_define
class JobsSinglePostResponseDataAttributes:
    """
    Attributes:
        job_id (Union[Unset, str]):  Example: example.
        name (Union[Unset, str]):  Example: example.
        state (Union[Unset, str]):  Example: example.
        status (Union[Unset, JobsSinglePostResponseDataAttributesStatus]):
    """

    job_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    status: Union[Unset, "JobsSinglePostResponseDataAttributesStatus"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        job_id = self.job_id

        name = self.name

        state = self.state

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if job_id is not UNSET:
            field_dict["jobId"] = job_id
        if name is not UNSET:
            field_dict["name"] = name
        if state is not UNSET:
            field_dict["state"] = state
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.jobs_single_post_response_data_attributes_status import (
            JobsSinglePostResponseDataAttributesStatus,
        )

        d = src_dict.copy()
        job_id = d.pop("jobId", UNSET)

        name = d.pop("name", UNSET)

        state = d.pop("state", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, JobsSinglePostResponseDataAttributesStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = JobsSinglePostResponseDataAttributesStatus.from_dict(
                _status
            )

        jobs_single_post_response_data_attributes_obj = cls(
            job_id=job_id,
            name=name,
            state=state,
            status=status,
        )

        jobs_single_post_response_data_attributes_obj.additional_properties = d
        return jobs_single_post_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.teststeps_single_get_response_data_attributes_values_item import (
        TeststepsSingleGetResponseDataAttributesValuesItem,
    )


T = TypeVar("T", bound="TeststepsSingleGetResponseDataAttributes")


@_attrs_define
class TeststepsSingleGetResponseDataAttributes:
    """
    Attributes:
        index (Union[Unset, str]):
        keys (Union[Unset, List[str]]):
        values (Union[Unset, List['TeststepsSingleGetResponseDataAttributesValuesItem']]):
    """

    index: Union[Unset, str] = UNSET
    keys: Union[Unset, List[str]] = UNSET
    values: Union[
        Unset, List["TeststepsSingleGetResponseDataAttributesValuesItem"]
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        index = self.index

        keys: Union[Unset, List[str]] = UNSET
        if not isinstance(self.keys, Unset):
            keys = self.keys

        values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.values, Unset):
            values = []
            for values_item_data in self.values:
                values_item = values_item_data.to_dict()
                values.append(values_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if index is not UNSET:
            field_dict["index"] = index
        if keys is not UNSET:
            field_dict["keys"] = keys
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.teststeps_single_get_response_data_attributes_values_item import (
            TeststepsSingleGetResponseDataAttributesValuesItem,
        )

        d = src_dict.copy()
        index = d.pop("index", UNSET)

        keys = cast(List[str], d.pop("keys", UNSET))

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:
            values_item = (
                TeststepsSingleGetResponseDataAttributesValuesItem.from_dict(
                    values_item_data
                )
            )

            values.append(values_item)

        teststeps_single_get_response_data_attributes_obj = cls(
            index=index,
            keys=keys,
            values=values,
        )

        teststeps_single_get_response_data_attributes_obj.additional_properties = (
            d
        )
        return teststeps_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document_comments_list_get_response_data_item_attributes_text import (
        DocumentCommentsListGetResponseDataItemAttributesText,
    )


T = TypeVar("T", bound="DocumentCommentsListGetResponseDataItemAttributes")


@_attrs_define
class DocumentCommentsListGetResponseDataItemAttributes:
    """
    Attributes:
        created (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        id (Union[Unset, str]):  Example: MyCommentId.
        resolved (Union[Unset, bool]):
        text (Union[Unset, DocumentCommentsListGetResponseDataItemAttributesText]):
    """

    created: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    resolved: Union[Unset, bool] = UNSET
    text: Union[
        Unset, "DocumentCommentsListGetResponseDataItemAttributesText"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        id = self.id

        resolved = self.resolved

        text: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.text, Unset):
            text = self.text.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created is not UNSET:
            field_dict["created"] = created
        if id is not UNSET:
            field_dict["id"] = id
        if resolved is not UNSET:
            field_dict["resolved"] = resolved
        if text is not UNSET:
            field_dict["text"] = text

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document_comments_list_get_response_data_item_attributes_text import (
            DocumentCommentsListGetResponseDataItemAttributesText,
        )

        d = src_dict.copy()
        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        id = d.pop("id", UNSET)

        resolved = d.pop("resolved", UNSET)

        _text = d.pop("text", UNSET)
        text: Union[
            Unset, DocumentCommentsListGetResponseDataItemAttributesText
        ]
        if isinstance(_text, Unset):
            text = UNSET
        else:
            text = DocumentCommentsListGetResponseDataItemAttributesText.from_dict(
                _text
            )

        document_comments_list_get_response_data_item_attributes_obj = cls(
            created=created,
            id=id,
            resolved=resolved,
            text=text,
        )

        document_comments_list_get_response_data_item_attributes_obj.additional_properties = (
            d
        )
        return document_comments_list_get_response_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Implementations of WorkItemAttachment relates functions."""
import io
import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.work_item_attachments import (  # pylint: disable=line-too-long
    delete_work_item_attachment,
    get_work_item_attachments,
    patch_work_item_attachment,
    post_work_item_attachments,
)

from . import base_classes as bc


class WorkItemAttachments(
    bc.SingleUpdatableItemsMixin[dm.WorkItemAttachment],
    bc.UpdatableItemsClient[dm.WorkItemAttachment],
):
    """A class to handle WorkItemAttachments."""

    def get(self, *args, **kwargs) -> dm.WorkItemAttachment:
        """Return a specific attachment - not Implemented yet."""
        raise NotImplementedError

    def _update(
        self, to_update: dm.WorkItemAttachment | list[dm.WorkItemAttachment]
    ):
        """Update the given work item attachment in Polarion."""
        assert not isinstance(to_update, list), "Expected only one item"
        attributes = (
            api_models.WorkitemAttachmentsSinglePatchRequestDataAttributes()
        )
        if to_update.title:
            attributes.title = to_update.title

        multipart = api_models.PatchWorkItemAttachmentsRequestBody(
            resource=api_models.WorkitemAttachmentsSinglePatchRequest(
                data=api_models.WorkitemAttachmentsSinglePatchRequestData(
                    type=api_models.WorkitemAttachmentsSinglePatchRequestDataType.WORKITEM_ATTACHMENTS,  # pylint: disable=line-too-long
                    id=f"{self._project_id}/{to_update.work_item_id}/{to_update.id}",  # pylint: disable=line-too-long
                    attributes=attributes,
                )
            )
        )

        if to_update.content_bytes:
            multipart.content = oa_types.File(
                io.BytesIO(to_update.content_bytes),
                to_update.file_name,
                to_update.mime_type,
            )

        response = patch_work_item_attachment.sync_detailed(
            self._project_id,
            to_update.work_item_id,
            to_update.id,
            client=self._client.client,
            body=multipart,
        )
        self._raise_on_error(response)

    def get_multi(  # type: ignore[override]
        self,
        work_item_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: t.Optional[dict[str, str]] = None,
    ) -> tuple[list[dm.WorkItemAttachment], bool]:
        """Return the attachments for a given work item on a defined page.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        if fields is None:
            fields = self._client.default_fields.workitem_attachments

        sparse_fields = self._build_sparse_fields(fields)
        response = get_work_item_attachments.sync_detailed(
            self._project_id,
            work_item_id=work_item_id,
            client=self._client.client,
            fields=sparse_fields,
            pagesize=page_size,
            pagenumber=page_number,
        )

        self._raise_on_error(response)

        parsed_response = response.parsed

        work_item_attachments: list[dm.WorkItemAttachment] = []

        next_page = False

        if (
            isinstance(
                parsed_response, api_models.WorkitemAttachmentsListGetResponse
            )
            and parsed_response.data
        ):
            for attachment in parsed_response.data:
                assert attachment.attributes
                assert isinstance(attachment.attributes.id, str)

                work_item_attachments.append(
                    dm.WorkItemAttachment(
                        work_item_id,
                        attachment.attributes.id,
                        self.unset_to_none(attachment.attributes.title),
                        file_name=self.unset_to_none(
                            attachment.attributes.file_name
                        ),
                    )
                )

            next_page = isinstance(
                parsed_response.links,
                api_models.WorkitemAttachmentsListGetResponseLinks,
            ) and bool(parsed_response.links.next_)

        return work_item_attachments, next_page

    def _create(self, items: list[dm.WorkItemAttachment]):
        """Create the given work item attachment in Polarion."""
        attachment_attributes = []
        attachment_files = []
        assert len(items), "No attachments were provided."
        assert all(
            [wia.work_item_id == items[0].work_item_id] for wia in items
        ), "All attachments must belong to the same WorkItem."
        for work_item_attachment in items:
            assert (
                work_item_attachment.file_name
            ), "You have to define a FileName."
            assert (
                work_item_attachment.content_bytes
            ), "You have to provide content bytes."
            assert (
                work_item_attachment.mime_type
            ), "You have to provide a mime_type."

            attributes = api_models.WorkitemAttachmentsListPostRequestDataItemAttributes(  # pylint: disable=line-too-long
                file_name=work_item_attachment.file_name
            )
            if work_item_attachment.title:
                attributes.title = work_item_attachment.title

            attachment_attributes.append(
                api_models.WorkitemAttachmentsListPostRequestDataItem(
                    type=api_models.WorkitemAttachmentsListPostRequestDataItemType.WORKITEM_ATTACHMENTS,  # pylint: disable=line-too-long
                    attributes=attributes,
                )
            )

            attachment_files.append(
                oa_types.File(
                    io.BytesIO(work_item_attachment.content_bytes),
                    work_item_attachment.file_name,
                    work_item_attachment.mime_type,
                )
            )
        multipart = api_models.PostWorkItemAttachmentsRequestBody(
            resource=api_models.WorkitemAttachmentsListPostRequest(
                attachment_attributes
            ),
            files=attachment_files,
        )
        response = post_work_item_attachments.sync_detailed(
            self._project_id,
            items[0].work_item_id,
            client=self._client.client,
            body=multipart,
        )

        self._raise_on_error(response)
        assert (
            isinstance(
                response.parsed, api_models.WorkitemAttachmentsListPostResponse
            )
            and response.parsed.data
        )
        counter = 0
        for work_item_attachment_res in response.parsed.data:
            assert work_item_attachment_res.id
            items[counter].id = work_item_attachment_res.id.split("/")[-1]
            counter += 1

    def _delete(self, items: list[dm.WorkItemAttachment]):
        for item in items:
            self._retry_on_error(self._single_delete, item)

    def _single_delete(self, work_item_attachment: dm.WorkItemAttachment):
        """Delete the given work item attachment."""
        response = delete_work_item_attachment.sync_detailed(
            self._project_id,
            work_item_attachment.work_item_id,
            work_item_attachment.id,
            client=self._client.client,
        )
        self._raise_on_error(response)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.plans_list_post_request_data_item_relationships_parent import (
        PlansListPostRequestDataItemRelationshipsParent,
    )
    from ..models.plans_list_post_request_data_item_relationships_project_span import (
        PlansListPostRequestDataItemRelationshipsProjectSpan,
    )
    from ..models.plans_list_post_request_data_item_relationships_template import (
        PlansListPostRequestDataItemRelationshipsTemplate,
    )
    from ..models.plans_list_post_request_data_item_relationships_work_items import (
        PlansListPostRequestDataItemRelationshipsWorkItems,
    )


T = TypeVar("T", bound="PlansListPostRequestDataItemRelationships")


@_attrs_define
class PlansListPostRequestDataItemRelationships:
    """
    Attributes:
        parent (Union[Unset, PlansListPostRequestDataItemRelationshipsParent]):
        project_span (Union[Unset, PlansListPostRequestDataItemRelationshipsProjectSpan]):
        template (Union[Unset, PlansListPostRequestDataItemRelationshipsTemplate]):
        work_items (Union[Unset, PlansListPostRequestDataItemRelationshipsWorkItems]):
    """

    parent: Union[Unset, "PlansListPostRequestDataItemRelationshipsParent"] = (
        UNSET
    )
    project_span: Union[
        Unset, "PlansListPostRequestDataItemRelationshipsProjectSpan"
    ] = UNSET
    template: Union[
        Unset, "PlansListPostRequestDataItemRelationshipsTemplate"
    ] = UNSET
    work_items: Union[
        Unset, "PlansListPostRequestDataItemRelationshipsWorkItems"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        project_span: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project_span, Unset):
            project_span = self.project_span.to_dict()

        template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.template, Unset):
            template = self.template.to_dict()

        work_items: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.work_items, Unset):
            work_items = self.work_items.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if parent is not UNSET:
            field_dict["parent"] = parent
        if project_span is not UNSET:
            field_dict["projectSpan"] = project_span
        if template is not UNSET:
            field_dict["template"] = template
        if work_items is not UNSET:
            field_dict["workItems"] = work_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.plans_list_post_request_data_item_relationships_parent import (
            PlansListPostRequestDataItemRelationshipsParent,
        )
        from ..models.plans_list_post_request_data_item_relationships_project_span import (
            PlansListPostRequestDataItemRelationshipsProjectSpan,
        )
        from ..models.plans_list_post_request_data_item_relationships_template import (
            PlansListPostRequestDataItemRelationshipsTemplate,
        )
        from ..models.plans_list_post_request_data_item_relationships_work_items import (
            PlansListPostRequestDataItemRelationshipsWorkItems,
        )

        d = src_dict.copy()
        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, PlansListPostRequestDataItemRelationshipsParent]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = PlansListPostRequestDataItemRelationshipsParent.from_dict(
                _parent
            )

        _project_span = d.pop("projectSpan", UNSET)
        project_span: Union[
            Unset, PlansListPostRequestDataItemRelationshipsProjectSpan
        ]
        if isinstance(_project_span, Unset):
            project_span = UNSET
        else:
            project_span = (
                PlansListPostRequestDataItemRelationshipsProjectSpan.from_dict(
                    _project_span
                )
            )

        _template = d.pop("template", UNSET)
        template: Union[
            Unset, PlansListPostRequestDataItemRelationshipsTemplate
        ]
        if isinstance(_template, Unset):
            template = UNSET
        else:
            template = (
                PlansListPostRequestDataItemRelationshipsTemplate.from_dict(
                    _template
                )
            )

        _work_items = d.pop("workItems", UNSET)
        work_items: Union[
            Unset, PlansListPostRequestDataItemRelationshipsWorkItems
        ]
        if isinstance(_work_items, Unset):
            work_items = UNSET
        else:
            work_items = (
                PlansListPostRequestDataItemRelationshipsWorkItems.from_dict(
                    _work_items
                )
            )

        plans_list_post_request_data_item_relationships_obj = cls(
            parent=parent,
            project_span=project_span,
            template=template,
            work_items=work_items,
        )

        plans_list_post_request_data_item_relationships_obj.additional_properties = (
            d
        )
        return plans_list_post_request_data_item_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.users_single_patch_request_data_attributes_description import (
        UsersSinglePatchRequestDataAttributesDescription,
    )


T = TypeVar("T", bound="UsersSinglePatchRequestDataAttributes")


@_attrs_define
class UsersSinglePatchRequestDataAttributes:
    """
    Attributes:
        description (Union[Unset, UsersSinglePatchRequestDataAttributesDescription]):
        disabled_notifications (Union[Unset, bool]):
        email (Union[Unset, str]):  Example: Email.
        initials (Union[Unset, str]):  Example: Initials.
        name (Union[Unset, str]):  Example: Name.
    """

    description: Union[
        Unset, "UsersSinglePatchRequestDataAttributesDescription"
    ] = UNSET
    disabled_notifications: Union[Unset, bool] = UNSET
    email: Union[Unset, str] = UNSET
    initials: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        disabled_notifications = self.disabled_notifications

        email = self.email

        initials = self.initials

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if disabled_notifications is not UNSET:
            field_dict["disabledNotifications"] = disabled_notifications
        if email is not UNSET:
            field_dict["email"] = email
        if initials is not UNSET:
            field_dict["initials"] = initials
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.users_single_patch_request_data_attributes_description import (
            UsersSinglePatchRequestDataAttributesDescription,
        )

        d = src_dict.copy()
        _description = d.pop("description", UNSET)
        description: Union[
            Unset, UsersSinglePatchRequestDataAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = (
                UsersSinglePatchRequestDataAttributesDescription.from_dict(
                    _description
                )
            )

        disabled_notifications = d.pop("disabledNotifications", UNSET)

        email = d.pop("email", UNSET)

        initials = d.pop("initials", UNSET)

        name = d.pop("name", UNSET)

        users_single_patch_request_data_attributes_obj = cls(
            description=description,
            disabled_notifications=disabled_notifications,
            email=email,
            initials=initials,
            name=name,
        )

        users_single_patch_request_data_attributes_obj.additional_properties = (
            d
        )
        return users_single_patch_request_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import pathlib

import pytest

import polarion_rest_api_client as polarion_api
from polarion_rest_api_client.clients import base_classes


@pytest.fixture(name="client")
def fixture_client():
    base_classes._max_sleep = 0
    base_classes._min_sleep = 0
    client = polarion_api.PolarionClient(
        polarion_api_endpoint="http://127.0.0.1/api",
        polarion_access_token="PAT123",
        batch_size=3,
    )
    yield client.generate_project_client(
        project_id="PROJ", delete_status="deleted"
    )


@pytest.fixture(name="work_item")
def fixture_dummy_work_item():
    return polarion_api.WorkItem(
        title="Title",
        description=polarion_api.HtmlContent("My text value"),
        type="task",
        status="open",
        additional_attributes={"capella_uuid": "asdfg"},
    )


@pytest.fixture(name="work_item_patch")
def fixture_dummy_work_item_patch():
    return polarion_api.WorkItem(
        id="MyWorkItemId",
        title="Title",
        description=polarion_api.HtmlContent("My text value"),
        status="open",
        additional_attributes={"capella_uuid": "qwertz"},
    )


@pytest.fixture(name="work_item_attachment")
def fixture_dummy_work_item_attachment():
    with open(TEST_WIA_CREATED_RESPONSE, encoding="utf8") as f:
        attachment_content = f.read()

    return polarion_api.WorkItemAttachment(
        "MyWorkItemId",
        "Attachment",
        "Title",
        attachment_content.encode("utf-8"),
        "text/plain",
        "test.json",
    )


TEST_DATA_ROOT = pathlib.Path(__file__).parent / "data"
TEST_RESPONSES = TEST_DATA_ROOT / "mock_api_responses"
TEST_REQUESTS = TEST_DATA_ROOT / "expected_requests"
TEST_WIA_NEXT_PAGE_RESPONSE = (
    TEST_RESPONSES / "get_work_item_attachments_next_page.json"
)
TEST_WIA_NO_NEXT_PAGE_RESPONSE = (
    TEST_RESPONSES / "get_work_item_attachments_no_next_page.json"
)
TEST_WIA_CREATED_RESPONSE = (
    TEST_RESPONSES / "created_work_item_attachment.json"
)
TEST_WIA_MULTI_CREATED_RESPONSE = (
    TEST_RESPONSES / "created_work_item_attachments.json"
)
TEST_WIL_MULTI_POST_REQUEST = TEST_REQUESTS / "post_work_item_links.json"
TEST_WIL_DELETE2_REQUEST = TEST_REQUESTS / "delete_work_item_link_2.json"
TEST_WIL_DELETE_REQUEST = TEST_REQUESTS / "delete_work_item_links.json"
TEST_WIL_DELETED_REQUEST = TEST_REQUESTS / "delete_work_item_link.json"
TEST_WIL_POSTED_REQUEST = TEST_REQUESTS / "post_work_item_link.json"
TEST_WIL_CREATED_RESPONSE = TEST_RESPONSES / "created_work_item_links.json"
TEST_WIL_NEXT_PAGE_RESPONSE = (
    TEST_RESPONSES / "get_linked_work_items_next_page.json"
)
TEST_WIL_NO_NEXT_PAGE_RESPONSE = (
    TEST_RESPONSES / "get_linked_work_items_no_next_page.json"
)
TEST_WI_DELETE_REQUEST = TEST_REQUESTS / "delete_work_item.json"
TEST_WI_PATCH_STATUS_DELETED_REQUEST = (
    TEST_REQUESTS / "patch_work_item_status_deleted.json"
)
TEST_WI_PATCH_STATUS_REQUEST = TEST_REQUESTS / "patch_work_item_status.json"
TEST_WI_PATCH_TITLE_REQUEST = TEST_REQUESTS / "patch_work_item_title.json"
TEST_WI_PATCH_DESCRIPTION_REQUEST = (
    TEST_REQUESTS / "patch_work_item_description.json"
)
TEST_WI_PATCH_COMPLETELY_REQUEST = (
    TEST_REQUESTS / "patch_work_item_completely.json"
)
TEST_WI_MULTI_POST_REQUEST = TEST_REQUESTS / "post_workitems.json"
TEST_WI_MULTI_POST_REQUEST_IN_DOC = TEST_REQUESTS / "post_workitem_in_doc.json"
TEST_WI_POST_REQUEST = TEST_REQUESTS / "post_workitem.json"
TEST_WI_NO_NEXT_PAGE_RESPONSE = TEST_RESPONSES / "workitems_no_next_page.json"
TEST_WI_CREATED_RESPONSE = TEST_RESPONSES / "created_work_items.json"
TEST_WI_ERROR_NEXT_PAGE_RESPONSE = (
    TEST_RESPONSES / "workitems_next_page_error.json"
)
TEST_WI_NEXT_PAGE_RESPONSE = TEST_RESPONSES / "workitems_next_page.json"
TEST_WI_SINGLE_RESPONSE = TEST_RESPONSES / "get_work_item.json"
TEST_WI_NOT_TRUNCATED_RESPONSE = (
    TEST_RESPONSES / "get_work_item_not_truncated.json"
)
TEST_DOCUMENT_RESPONSE = TEST_RESPONSES / "get_document.json"
TEST_DOCUMENT_POST_REQUEST = TEST_REQUESTS / "create_document.json"
TEST_DOCUMENT_PATCH_REQUEST = TEST_REQUESTS / "update_document.json"
TEST_DOCUMENT_PATCH_REQUEST2 = TEST_REQUESTS / "update_document_2.json"
TEST_ERROR_RESPONSE = TEST_RESPONSES / "error.json"
TEST_FAULTS_ERROR_RESPONSES = TEST_RESPONSES / "faulty_errors.json"
TEST_PROJECT_RESPONSE_JSON = TEST_RESPONSES / "project.json"
TEST_TRUN_PATCH_REQUEST = TEST_REQUESTS / "patch_test_run_partially.json"
TEST_TRUN_FULLY_PATCH_REQUEST = TEST_REQUESTS / "patch_test_run_fully.json"
TEST_TRUN_POST_REQUEST = TEST_REQUESTS / "post_test_run.json"
TEST_TREC_PATCH_REQUEST = TEST_REQUESTS / "patch_test_record.json"
TEST_TREC_POST_REQUEST = TEST_REQUESTS / "post_test_records.json"
TEST_TREC_CREATED_RESPONSE = TEST_RESPONSES / "created_test_records.json"
TEST_TRUN_CREATED_RESPONSE = TEST_RESPONSES / "created_test_runs.json"
TEST_TREC_NEXT_RESPONSE = TEST_RESPONSES / "test_records_next_page.json"
TEST_TREC_NO_NEXT_RESPONSE = TEST_RESPONSES / "test_records_no_next_page.json"
TEST_TRUN_NEXT_RESPONSE = TEST_RESPONSES / "test_runs_next_page.json"
TEST_TRUN_NO_NEXT_RESPONSE = TEST_RESPONSES / "test_runs_no_next_page.json"


class CustomWorkItem(polarion_api.WorkItem):
    capella_uuid: str | None



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.enum_options_action_response_body import (
    EnumOptionsActionResponseBody,
)
from ...models.errors import Errors
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    space_id: str,
    document_name: str,
    field_id: str,
    *,
    pagesize: Union[Unset, int] = UNSET,
    pagenumber: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["page[size]"] = pagesize

    params["page[number]"] = pagenumber

    params = {
        k: v for k, v in params.items() if v is not UNSET and v is not None
    }

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/projects/{projectId}/spaces/{spaceId}/documents/{documentName}/fields/{fieldId}/actions/getAvailableOptions".format(
            projectId=project_id,
            spaceId=space_id,
            documentName=document_name,
            fieldId=field_id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[EnumOptionsActionResponseBody, Errors]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = EnumOptionsActionResponseBody.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Errors.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Errors.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Errors.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Errors.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.NOT_ACCEPTABLE:
        response_406 = Errors.from_dict(response.json())

        return response_406
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Errors.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.SERVICE_UNAVAILABLE:
        response_503 = Errors.from_dict(response.json())

        return response_503
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[EnumOptionsActionResponseBody, Errors]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    space_id: str,
    document_name: str,
    field_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    pagesize: Union[Unset, int] = UNSET,
    pagenumber: Union[Unset, int] = UNSET,
) -> Response[Union[EnumOptionsActionResponseBody, Errors]]:
    """Returns a list of available options for the requested field in the
    specified Document.

    Args:
        project_id (str):
        space_id (str):
        document_name (str):
        field_id (str):
        pagesize (Union[Unset, int]):
        pagenumber (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[EnumOptionsActionResponseBody, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        space_id=space_id,
        document_name=document_name,
        field_id=field_id,
        pagesize=pagesize,
        pagenumber=pagenumber,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    space_id: str,
    document_name: str,
    field_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    pagesize: Union[Unset, int] = UNSET,
    pagenumber: Union[Unset, int] = UNSET,
) -> Optional[Union[EnumOptionsActionResponseBody, Errors]]:
    """Returns a list of available options for the requested field in the
    specified Document.

    Args:
        project_id (str):
        space_id (str):
        document_name (str):
        field_id (str):
        pagesize (Union[Unset, int]):
        pagenumber (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[EnumOptionsActionResponseBody, Errors]
    """

    return sync_detailed(
        project_id=project_id,
        space_id=space_id,
        document_name=document_name,
        field_id=field_id,
        client=client,
        pagesize=pagesize,
        pagenumber=pagenumber,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    space_id: str,
    document_name: str,
    field_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    pagesize: Union[Unset, int] = UNSET,
    pagenumber: Union[Unset, int] = UNSET,
) -> Response[Union[EnumOptionsActionResponseBody, Errors]]:
    """Returns a list of available options for the requested field in the
    specified Document.

    Args:
        project_id (str):
        space_id (str):
        document_name (str):
        field_id (str):
        pagesize (Union[Unset, int]):
        pagenumber (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[EnumOptionsActionResponseBody, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        space_id=space_id,
        document_name=document_name,
        field_id=field_id,
        pagesize=pagesize,
        pagenumber=pagenumber,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    space_id: str,
    document_name: str,
    field_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    pagesize: Union[Unset, int] = UNSET,
    pagenumber: Union[Unset, int] = UNSET,
) -> Optional[Union[EnumOptionsActionResponseBody, Errors]]:
    """Returns a list of available options for the requested field in the
    specified Document.

    Args:
        project_id (str):
        space_id (str):
        document_name (str):
        field_id (str):
        pagesize (Union[Unset, int]):
        pagenumber (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[EnumOptionsActionResponseBody, Errors]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            space_id=space_id,
            document_name=document_name,
            field_id=field_id,
            client=client,
            pagesize=pagesize,
            pagenumber=pagenumber,
        )
    ).parsed



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.set_license_request_body_license import (
    SetLicenseRequestBodyLicense,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="SetLicenseRequestBody")


@_attrs_define
class SetLicenseRequestBody:
    """
    Attributes:
        concurrent (Union[Unset, bool]): Is concurrent user Example: True.
        group (Union[Unset, str]): License group Example: Department.
        license_ (Union[Unset, SetLicenseRequestBodyLicense]): User's license type
    """

    concurrent: Union[Unset, bool] = UNSET
    group: Union[Unset, str] = UNSET
    license_: Union[Unset, SetLicenseRequestBodyLicense] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        concurrent = self.concurrent

        group = self.group

        license_: Union[Unset, str] = UNSET
        if not isinstance(self.license_, Unset):
            license_ = self.license_.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if concurrent is not UNSET:
            field_dict["concurrent"] = concurrent
        if group is not UNSET:
            field_dict["group"] = group
        if license_ is not UNSET:
            field_dict["license"] = license_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        concurrent = d.pop("concurrent", UNSET)

        group = d.pop("group", UNSET)

        _license_ = d.pop("license", UNSET)
        license_: Union[Unset, SetLicenseRequestBodyLicense]
        if isinstance(_license_, Unset):
            license_ = UNSET
        else:
            license_ = SetLicenseRequestBodyLicense(_license_)

        set_license_request_body_obj = cls(
            concurrent=concurrent,
            group=group,
            license_=license_,
        )

        set_license_request_body_obj.additional_properties = d
        return set_license_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_project_request_body_params_type_0 import (
        CreateProjectRequestBodyParamsType0,
    )


T = TypeVar("T", bound="CreateProjectRequestBody")


@_attrs_define
class CreateProjectRequestBody:
    """
    Attributes:
        location (Union[Unset, str]): Location of the new Project to be created. Example: MyLocation.
        params (Union['CreateProjectRequestBodyParamsType0', None, Unset]): params of new Project to be created.
        project_id (Union[Unset, str]): Id of the new Project to be created. Example: MyProjectId.
        template_id (Union[None, Unset, str]): Id of the template to create the new Project from. Example:
            MyProjectTemplateId.
        tracker_prefix (Union[Unset, str]): Tracker prefix of the new Project to be created. Example: MyTrackerPrefix.
    """

    location: Union[Unset, str] = UNSET
    params: Union["CreateProjectRequestBodyParamsType0", None, Unset] = UNSET
    project_id: Union[Unset, str] = UNSET
    template_id: Union[None, Unset, str] = UNSET
    tracker_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        from ..models.create_project_request_body_params_type_0 import (
            CreateProjectRequestBodyParamsType0,
        )

        location = self.location

        params: Union[Dict[str, Any], None, Unset]
        if isinstance(self.params, Unset):
            params = UNSET
        elif isinstance(self.params, CreateProjectRequestBodyParamsType0):
            params = self.params.to_dict()
        else:
            params = self.params

        project_id = self.project_id

        template_id: Union[None, Unset, str]
        if isinstance(self.template_id, Unset):
            template_id = UNSET
        else:
            template_id = self.template_id

        tracker_prefix = self.tracker_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if location is not UNSET:
            field_dict["location"] = location
        if params is not UNSET:
            field_dict["params"] = params
        if project_id is not UNSET:
            field_dict["projectId"] = project_id
        if template_id is not UNSET:
            field_dict["templateId"] = template_id
        if tracker_prefix is not UNSET:
            field_dict["trackerPrefix"] = tracker_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_project_request_body_params_type_0 import (
            CreateProjectRequestBodyParamsType0,
        )

        d = src_dict.copy()
        location = d.pop("location", UNSET)

        def _parse_params(
            data: object,
        ) -> Union["CreateProjectRequestBodyParamsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                params_type_0 = CreateProjectRequestBodyParamsType0.from_dict(
                    data
                )

                return params_type_0
            except:  # noqa: E722
                pass
            return cast(
                Union["CreateProjectRequestBodyParamsType0", None, Unset], data
            )

        params = _parse_params(d.pop("params", UNSET))

        project_id = d.pop("projectId", UNSET)

        def _parse_template_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        template_id = _parse_template_id(d.pop("templateId", UNSET))

        tracker_prefix = d.pop("trackerPrefix", UNSET)

        create_project_request_body_obj = cls(
            location=location,
            params=params,
            project_id=project_id,
            template_id=template_id,
            tracker_prefix=tracker_prefix,
        )

        create_project_request_body_obj.additional_properties = d
        return create_project_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import json

import pytest
import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import TEST_FAULTS_ERROR_RESPONSES


def test_faulty_error_message(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_FAULTS_ERROR_RESPONSES, encoding="utf8") as f:
        response = json.load(f)

    httpx_mock.add_response(400, json=response)
    httpx_mock.add_response(400, json=response)

    with pytest.raises(polarion_api.PolarionApiException) as e_info:
        client.documents.get(
            "MySpaceId", "MyDocumentName", {"fields[documents]": "@all"}
        )

    e = e_info.value
    assert len(e.args) == 6
    assert len(httpx_mock.get_requests()) == 2
    assert e.args[1][0] == "400"
    assert (
        e.args[1][1] == "Unexpected token, BEGIN_ARRAY expected, but was : "
        "BEGIN_OBJECT (at $.data)"
    )


def test_dont_retry_on_404(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_FAULTS_ERROR_RESPONSES, encoding="utf8") as f:
        httpx_mock.add_response(404, json=json.load(f))

    with pytest.raises(polarion_api.PolarionApiException) as e_info:
        client.documents.get(
            "MySpaceId", "MyDocumentName", {"fields[documents]": "@all"}
        )

    e = e_info.value
    assert len(httpx_mock.get_requests()) == 1
    assert e.args[0] == 404



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""The overall Polarion Client which handles the HTTP session, auth etc."""
from __future__ import annotations

import typing as t

import polarion_rest_api_client.open_api_client as oa_client
from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.clients import projects

WorkItemType = t.TypeVar("WorkItemType", bound=dm.WorkItem)


class DefaultFields:
    """A class to define default values for the fields parameter."""

    _workitems: str = "@basic"
    _linkedworkitems: str = "id,role,suspect"
    _workitem_attachments: str = "@basic"
    _documents: str = "@basic"
    _testrecords: str = "@basic"
    _testruns: str = "@basic"

    @property
    def workitems(self):
        """Return the fields dict for workitems."""
        return {"workitems": self._workitems}

    @workitems.setter
    def workitems(self, value):
        self._workitems = value

    @property
    def linkedworkitems(self):
        """Return the fields dict for linkedworkitems."""
        return {"linkedworkitems": self._linkedworkitems}

    @linkedworkitems.setter
    def linkedworkitems(self, value):
        self._linkedworkitems = value

    @property
    def workitem_attachments(self):
        """Return the fields dict for workitem_attachments."""
        return {"workitem_attachments": self._workitem_attachments}

    @workitem_attachments.setter
    def workitem_attachments(self, value):
        self._workitem_attachments = value

    @property
    def documents(self):
        """Return the fields dict for document."""
        return {"documents": self._documents}

    @documents.setter
    def documents(self, value):
        self._documents = value

    @property
    def testruns(self):
        """Return the fields dict for document."""
        return {"testruns": self._testruns}

    @testruns.setter
    def testruns(self, value):
        self._testruns = value

    @property
    def testrecords(self):
        """Return the fields dict for document."""
        return {"testrecords": self._testrecords}

    @testrecords.setter
    def testrecords(self, value):
        self._testrecords = value

    @property
    def all_types(self):
        """Return all fields dicts merged together."""
        return (
            self.workitem_attachments
            | self.workitems
            | self.linkedworkitems
            | self.documents
            | self.testruns
            | self.testrecords
        )


class PolarionClient:
    """The overall, project independent Polarion client."""

    def __init__(
        self,
        polarion_api_endpoint: str,
        polarion_access_token: str,
        httpx_args: t.Optional[dict[str, t.Any]] = None,
        batch_size: int = 100,
        page_size: int = 100,
        max_content_size: int = 2 * 1024**2,
    ):
        self.client = oa_client.AuthenticatedClient(
            polarion_api_endpoint,
            polarion_access_token,
            httpx_args=httpx_args or {},
        )
        self.batch_size = batch_size
        self.page_size = page_size
        self.max_content_size = max_content_size
        self.default_fields = DefaultFields()

    def generate_project_client(
        self,
        project_id: str,
        delete_status: str | None = None,
    ):
        """Return a client for a specific project."""
        return projects.ProjectClient(project_id, self, delete_status)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.errors_errors_item_source_type_0_resource_type_0 import (
        ErrorsErrorsItemSourceType0ResourceType0,
    )


T = TypeVar("T", bound="ErrorsErrorsItemSourceType0")


@_attrs_define
class ErrorsErrorsItemSourceType0:
    """
    Attributes:
        parameter (Union[Unset, str]): String indicating which URI query parameter caused the error. Example: revision.
        pointer (Union[Unset, str]): JSON Pointer to the associated entity in the request document. Example: $.data.
        resource (Union['ErrorsErrorsItemSourceType0ResourceType0', None, Unset]): Resource causing the error.
    """

    parameter: Union[Unset, str] = UNSET
    pointer: Union[Unset, str] = UNSET
    resource: Union[
        "ErrorsErrorsItemSourceType0ResourceType0", None, Unset
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        from ..models.errors_errors_item_source_type_0_resource_type_0 import (
            ErrorsErrorsItemSourceType0ResourceType0,
        )

        parameter = self.parameter

        pointer = self.pointer

        resource: Union[Dict[str, Any], None, Unset]
        if isinstance(self.resource, Unset):
            resource = UNSET
        elif isinstance(
            self.resource, ErrorsErrorsItemSourceType0ResourceType0
        ):
            resource = self.resource.to_dict()
        else:
            resource = self.resource

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if parameter is not UNSET:
            field_dict["parameter"] = parameter
        if pointer is not UNSET:
            field_dict["pointer"] = pointer
        if resource is not UNSET:
            field_dict["resource"] = resource

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.errors_errors_item_source_type_0_resource_type_0 import (
            ErrorsErrorsItemSourceType0ResourceType0,
        )

        d = src_dict.copy()
        parameter = d.pop("parameter", UNSET)

        pointer = d.pop("pointer", UNSET)

        def _parse_resource(
            data: object,
        ) -> Union["ErrorsErrorsItemSourceType0ResourceType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                resource_type_0 = (
                    ErrorsErrorsItemSourceType0ResourceType0.from_dict(data)
                )

                return resource_type_0
            except:  # noqa: E722
                pass
            return cast(
                Union["ErrorsErrorsItemSourceType0ResourceType0", None, Unset],
                data,
            )

        resource = _parse_resource(d.pop("resource", UNSET))

        errors_errors_item_source_type_0_obj = cls(
            parameter=parameter,
            pointer=pointer,
            resource=resource,
        )

        errors_errors_item_source_type_0_obj.additional_properties = d
        return errors_errors_item_source_type_0_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.testruns_list_post_request_data_item_attributes_select_test_cases_by import (
    TestrunsListPostRequestDataItemAttributesSelectTestCasesBy,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.testruns_list_post_request_data_item_attributes_home_page_content import (
        TestrunsListPostRequestDataItemAttributesHomePageContent,
    )


T = TypeVar("T", bound="TestrunsListPostRequestDataItemAttributes")


@_attrs_define
class TestrunsListPostRequestDataItemAttributes:
    """
    Attributes:
        finished_on (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        group_id (Union[Unset, str]):  Example: Group ID.
        home_page_content (Union[Unset, TestrunsListPostRequestDataItemAttributesHomePageContent]):
        id (Union[Unset, str]):  Example: ID.
        id_prefix (Union[Unset, str]):  Example: MyTestRunIdPrefix.
        is_template (Union[Unset, bool]):
        keep_in_history (Union[Unset, bool]):
        query (Union[Unset, str]):  Example: Query.
        select_test_cases_by (Union[Unset, TestrunsListPostRequestDataItemAttributesSelectTestCasesBy]):  Example:
            manualSelection.
        status (Union[Unset, str]):  Example: open.
        title (Union[Unset, str]):  Example: Title.
        type (Union[Unset, str]):  Example: manual.
        use_report_from_template (Union[Unset, bool]):
    """

    finished_on: Union[Unset, datetime.datetime] = UNSET
    group_id: Union[Unset, str] = UNSET
    home_page_content: Union[
        Unset, "TestrunsListPostRequestDataItemAttributesHomePageContent"
    ] = UNSET
    id: Union[Unset, str] = UNSET
    id_prefix: Union[Unset, str] = UNSET
    is_template: Union[Unset, bool] = UNSET
    keep_in_history: Union[Unset, bool] = UNSET
    query: Union[Unset, str] = UNSET
    select_test_cases_by: Union[
        Unset, TestrunsListPostRequestDataItemAttributesSelectTestCasesBy
    ] = UNSET
    status: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    use_report_from_template: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        finished_on: Union[Unset, str] = UNSET
        if not isinstance(self.finished_on, Unset):
            finished_on = self.finished_on.isoformat()

        group_id = self.group_id

        home_page_content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.home_page_content, Unset):
            home_page_content = self.home_page_content.to_dict()

        id = self.id

        id_prefix = self.id_prefix

        is_template = self.is_template

        keep_in_history = self.keep_in_history

        query = self.query

        select_test_cases_by: Union[Unset, str] = UNSET
        if not isinstance(self.select_test_cases_by, Unset):
            select_test_cases_by = self.select_test_cases_by.value

        status = self.status

        title = self.title

        type = self.type

        use_report_from_template = self.use_report_from_template

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if finished_on is not UNSET:
            field_dict["finishedOn"] = finished_on
        if group_id is not UNSET:
            field_dict["groupId"] = group_id
        if home_page_content is not UNSET:
            field_dict["homePageContent"] = home_page_content
        if id is not UNSET:
            field_dict["id"] = id
        if id_prefix is not UNSET:
            field_dict["idPrefix"] = id_prefix
        if is_template is not UNSET:
            field_dict["isTemplate"] = is_template
        if keep_in_history is not UNSET:
            field_dict["keepInHistory"] = keep_in_history
        if query is not UNSET:
            field_dict["query"] = query
        if select_test_cases_by is not UNSET:
            field_dict["selectTestCasesBy"] = select_test_cases_by
        if status is not UNSET:
            field_dict["status"] = status
        if title is not UNSET:
            field_dict["title"] = title
        if type is not UNSET:
            field_dict["type"] = type
        if use_report_from_template is not UNSET:
            field_dict["useReportFromTemplate"] = use_report_from_template

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.testruns_list_post_request_data_item_attributes_home_page_content import (
            TestrunsListPostRequestDataItemAttributesHomePageContent,
        )

        d = src_dict.copy()
        _finished_on = d.pop("finishedOn", UNSET)
        finished_on: Union[Unset, datetime.datetime]
        if isinstance(_finished_on, Unset):
            finished_on = UNSET
        else:
            finished_on = isoparse(_finished_on)

        group_id = d.pop("groupId", UNSET)

        _home_page_content = d.pop("homePageContent", UNSET)
        home_page_content: Union[
            Unset, TestrunsListPostRequestDataItemAttributesHomePageContent
        ]
        if isinstance(_home_page_content, Unset):
            home_page_content = UNSET
        else:
            home_page_content = TestrunsListPostRequestDataItemAttributesHomePageContent.from_dict(
                _home_page_content
            )

        id = d.pop("id", UNSET)

        id_prefix = d.pop("idPrefix", UNSET)

        is_template = d.pop("isTemplate", UNSET)

        keep_in_history = d.pop("keepInHistory", UNSET)

        query = d.pop("query", UNSET)

        _select_test_cases_by = d.pop("selectTestCasesBy", UNSET)
        select_test_cases_by: Union[
            Unset, TestrunsListPostRequestDataItemAttributesSelectTestCasesBy
        ]
        if isinstance(_select_test_cases_by, Unset):
            select_test_cases_by = UNSET
        else:
            select_test_cases_by = (
                TestrunsListPostRequestDataItemAttributesSelectTestCasesBy(
                    _select_test_cases_by
                )
            )

        status = d.pop("status", UNSET)

        title = d.pop("title", UNSET)

        type = d.pop("type", UNSET)

        use_report_from_template = d.pop("useReportFromTemplate", UNSET)

        testruns_list_post_request_data_item_attributes_obj = cls(
            finished_on=finished_on,
            group_id=group_id,
            home_page_content=home_page_content,
            id=id,
            id_prefix=id_prefix,
            is_template=is_template,
            keep_in_history=keep_in_history,
            query=query,
            select_test_cases_by=select_test_cases_by,
            status=status,
            title=title,
            type=type,
            use_report_from_template=use_report_from_template,
        )

        testruns_list_post_request_data_item_attributes_obj.additional_properties = (
            d
        )
        return testruns_list_post_request_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class SetLicenseRequestBodyLicense(str, Enum):
    ALM = "ALM"
    PRO = "PRO"
    QA = "QA"
    REQUIREMENTS = "REQUIREMENTS"
    REVIEWER = "REVIEWER"
    XBASE = "XBase"
    XENTERPRISE = "XEnterprise"
    XPRO = "XPro"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import json

import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_WI_CREATED_RESPONSE,
    TEST_WI_NO_NEXT_PAGE_RESPONSE,
    TEST_WI_POST_REQUEST,
    CustomWorkItem,
)


def test_get_all_work_items_single_page_custom_work_item(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_NO_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    work_items = client.work_items.get_all("", work_item_cls=CustomWorkItem)

    assert isinstance(work_items[0], CustomWorkItem)
    assert work_items[0].capella_uuid == "asdfgh"


def test_create_work_item_custom_work_item(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))
    work_item = CustomWorkItem(
        title="Title",
        description=polarion_api.HtmlContent("My text value"),
        status="open",
        type="task",
        capella_uuid="asdfgh",
    )

    work_item.capella_uuid = "asdfg"

    client.work_items.create(work_item)

    req = httpx_mock.get_request()
    assert req.method == "POST"
    with open(TEST_WI_POST_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CopyDocumentRequestBody")


@_attrs_define
class CopyDocumentRequestBody:
    """
    Attributes:
        link_original_items_with_role (Union[Unset, str]): Link a copy of the document to the original. Example:
            duplicates.
        remove_outgoing_links (Union[Unset, bool]): Should outgoing links be removed? Example: True.
        target_document_name (Union[Unset, str]): Name for new Document. Example: MyDocumentId.
        target_project_id (Union[Unset, str]): Project where new document will be created. Example: MyProjectId.
        target_space_id (Union[Unset, str]): Space where new document will be created. Example: MySpaceId.
    """

    link_original_items_with_role: Union[Unset, str] = UNSET
    remove_outgoing_links: Union[Unset, bool] = UNSET
    target_document_name: Union[Unset, str] = UNSET
    target_project_id: Union[Unset, str] = UNSET
    target_space_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        link_original_items_with_role = self.link_original_items_with_role

        remove_outgoing_links = self.remove_outgoing_links

        target_document_name = self.target_document_name

        target_project_id = self.target_project_id

        target_space_id = self.target_space_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if link_original_items_with_role is not UNSET:
            field_dict["linkOriginalItemsWithRole"] = (
                link_original_items_with_role
            )
        if remove_outgoing_links is not UNSET:
            field_dict["removeOutgoingLinks"] = remove_outgoing_links
        if target_document_name is not UNSET:
            field_dict["targetDocumentName"] = target_document_name
        if target_project_id is not UNSET:
            field_dict["targetProjectId"] = target_project_id
        if target_space_id is not UNSET:
            field_dict["targetSpaceId"] = target_space_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        link_original_items_with_role = d.pop(
            "linkOriginalItemsWithRole", UNSET
        )

        remove_outgoing_links = d.pop("removeOutgoingLinks", UNSET)

        target_document_name = d.pop("targetDocumentName", UNSET)

        target_project_id = d.pop("targetProjectId", UNSET)

        target_space_id = d.pop("targetSpaceId", UNSET)

        copy_document_request_body_obj = cls(
            link_original_items_with_role=link_original_items_with_role,
            remove_outgoing_links=remove_outgoing_links,
            target_document_name=target_document_name,
            target_project_id=target_project_id,
            target_space_id=target_space_id,
        )

        copy_document_request_body_obj.additional_properties = d
        return copy_document_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import itertools
import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.test_records import (
    get_test_records,
    patch_test_record,
    post_test_records,
)

from . import base_classes as bc


class TestRecords(
    bc.SingleUpdatableItemsMixin[dm.TestRecord],
    bc.UpdatableItemsClient[dm.TestRecord],
):
    def get(self, *args, **kwargs) -> dm.TestRecord:
        raise NotImplementedError

    def _update(self, to_update: list[dm.TestRecord] | dm.TestRecord):
        assert not isinstance(to_update, list), "Expected only one item"
        response = patch_test_record.sync_detailed(
            self._project_id,
            to_update.test_run_id,
            to_update.work_item_project_id,
            to_update.work_item_id,
            str(to_update.iteration),
            client=self._client.client,
            # pylint: disable=line-too-long
            body=api_models.TestrecordsSinglePatchRequest(
                data=api_models.TestrecordsSinglePatchRequestData(
                    type=api_models.TestrecordsSinglePatchRequestDataType.TESTRECORDS,
                    id=f"{self._project_id}/{to_update.test_run_id}/{to_update.work_item_project_id}/{to_update.work_item_id}/{to_update.iteration}",
                    attributes=self._fill_test_record_attributes(
                        api_models.TestrecordsSinglePatchRequestDataAttributes,
                        to_update,
                    ),
                )
            ),
            # pylint: enable=line-too-long
        )
        self._raise_on_error(response)

    def get_multi(  # type: ignore[override]
        self,
        test_run_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
    ) -> tuple[list[dm.TestRecord], bool]:
        if fields is None:
            fields = self._client.default_fields.testrecords

        sparse_fields = self._build_sparse_fields(fields)
        response = get_test_records.sync_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        self._raise_on_error(response)

        parsed_response = response.parsed
        assert isinstance(
            parsed_response, api_models.TestrecordsListGetResponse
        )

        test_records = []

        for data in parsed_response.data or []:
            assert isinstance(data.id, str)
            assert isinstance(
                data.attributes,
                api_models.TestrecordsListGetResponseDataItemAttributes,
            )
            _, _, project_id, work_item, iteration = data.id.split("/")
            test_records.append(
                dm.TestRecord(
                    test_run_id,
                    project_id,
                    work_item,
                    self.unset_to_none(data.attributes.test_case_revision),
                    int(iteration),
                    (
                        data.attributes.duration
                        if not isinstance(
                            data.attributes.duration, oa_types.Unset
                        )
                        else -1
                    ),
                    self.unset_to_none(data.attributes.result),
                    self._handle_text_content(data.attributes.comment),
                    self.unset_to_none(data.attributes.executed),
                    data.additional_properties or {},
                )
            )
        next_page = isinstance(
            parsed_response.links,
            api_models.TestrecordsListGetResponseLinks,
        ) and bool(parsed_response.links.next_)

        return test_records, next_page

    def _split_into_batches(
        self, items: list[dm.TestRecord]
    ) -> t.Generator[list[dm.TestRecord], None, None]:
        for _, group in itertools.groupby(items, lambda x: x.test_run_id):
            yield from super()._split_into_batches(list(group))

    def _create(
        self,
        items: list[dm.TestRecord],
    ):
        """Create the given list of test records."""
        response = post_test_records.sync_detailed(
            self._project_id,
            items[0].test_run_id,
            client=self._client.client,
            # pylint: disable=line-too-long
            body=api_models.TestrecordsListPostRequest(
                [
                    api_models.TestrecordsListPostRequestDataItem(
                        type=api_models.TestrecordsListPostRequestDataItemType.TESTRECORDS,
                        attributes=self._fill_test_record_attributes(
                            api_models.TestrecordsListPostRequestDataItemAttributes,
                            test_record,
                        ),
                        relationships=api_models.TestrecordsListPostRequestDataItemRelationships(
                            test_case=api_models.TestrecordsListPostRequestDataItemRelationshipsTestCase(
                                data=api_models.TestrecordsListPostRequestDataItemRelationshipsTestCaseData(
                                    type=api_models.TestrecordsListPostRequestDataItemRelationshipsTestCaseDataType.WORKITEMS,
                                    id=f"{test_record.work_item_project_id}/{test_record.work_item_id}",
                                )
                            )
                        ),
                    )
                    for test_record in items
                ]
            ),
            # pylint: enable=line-too-long
        )

        self._raise_on_error(response)

        assert (
            isinstance(response.parsed, api_models.TestrecordsListPostResponse)
            and response.parsed.data
        )
        counter = 0
        for response_item in response.parsed.data:
            if response_item.id:
                items[counter].iteration = int(response_item.id.split("/")[-1])
                counter += 1

    def _delete(self, items: dm.TestRecord | list[dm.TestRecord]):
        raise NotImplementedError

    def _fill_test_record_attributes(
        self,
        attributes_type: type[
            api_models.TestrecordsListPostRequestDataItemAttributes
            | api_models.TestrecordsSinglePatchRequestDataAttributes
        ],
        test_record: dm.TestRecord,
    ):
        type_prefix = attributes_type.__name__
        attributes = attributes_type()
        if test_record.result:
            attributes.result = test_record.result
        if test_record.comment:
            attributes.comment = getattr(api_models, f"{type_prefix}Comment")()
            assert attributes.comment
            if test_record.comment.type:
                attributes.comment.type = getattr(
                    api_models, f"{type_prefix}CommentType"
                )(test_record.comment.type)
            if test_record.comment.value:
                attributes.comment.value = test_record.comment.value
        if test_record.duration != -1:
            attributes.duration = test_record.duration
        if test_record.work_item_revision:
            attributes.test_case_revision = test_record.work_item_revision
        if test_record.executed:
            attributes.executed = test_record.executed
        if test_record.additional_attributes:
            attributes.additional_properties = (
                test_record.additional_attributes
            )
        return attributes



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.workitem_approvals_list_get_response_data_item_attributes_status import (
    WorkitemApprovalsListGetResponseDataItemAttributesStatus,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkitemApprovalsListGetResponseDataItemAttributes")


@_attrs_define
class WorkitemApprovalsListGetResponseDataItemAttributes:
    """
    Attributes:
        status (Union[Unset, WorkitemApprovalsListGetResponseDataItemAttributesStatus]):  Example: waiting.
    """

    status: Union[
        Unset, WorkitemApprovalsListGetResponseDataItemAttributesStatus
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _status = d.pop("status", UNSET)
        status: Union[
            Unset, WorkitemApprovalsListGetResponseDataItemAttributesStatus
        ]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = WorkitemApprovalsListGetResponseDataItemAttributesStatus(
                _status
            )

        workitem_approvals_list_get_response_data_item_attributes_obj = cls(
            status=status,
        )

        workitem_approvals_list_get_response_data_item_attributes_obj.additional_properties = (
            d
        )
        return workitem_approvals_list_get_response_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.jobs_single_post_response_data_relationships_document import (
        JobsSinglePostResponseDataRelationshipsDocument,
    )
    from ..models.jobs_single_post_response_data_relationships_documents import (
        JobsSinglePostResponseDataRelationshipsDocuments,
    )
    from ..models.jobs_single_post_response_data_relationships_project import (
        JobsSinglePostResponseDataRelationshipsProject,
    )


T = TypeVar("T", bound="JobsSinglePostResponseDataRelationships")


@_attrs_define
class JobsSinglePostResponseDataRelationships:
    """
    Attributes:
        document (Union[Unset, JobsSinglePostResponseDataRelationshipsDocument]):
        documents (Union[Unset, JobsSinglePostResponseDataRelationshipsDocuments]):
        project (Union[Unset, JobsSinglePostResponseDataRelationshipsProject]):
    """

    document: Union[
        Unset, "JobsSinglePostResponseDataRelationshipsDocument"
    ] = UNSET
    documents: Union[
        Unset, "JobsSinglePostResponseDataRelationshipsDocuments"
    ] = UNSET
    project: Union[Unset, "JobsSinglePostResponseDataRelationshipsProject"] = (
        UNSET
    )
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        document: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.document, Unset):
            document = self.document.to_dict()

        documents: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.documents, Unset):
            documents = self.documents.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if document is not UNSET:
            field_dict["document"] = document
        if documents is not UNSET:
            field_dict["documents"] = documents
        if project is not UNSET:
            field_dict["project"] = project

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.jobs_single_post_response_data_relationships_document import (
            JobsSinglePostResponseDataRelationshipsDocument,
        )
        from ..models.jobs_single_post_response_data_relationships_documents import (
            JobsSinglePostResponseDataRelationshipsDocuments,
        )
        from ..models.jobs_single_post_response_data_relationships_project import (
            JobsSinglePostResponseDataRelationshipsProject,
        )

        d = src_dict.copy()
        _document = d.pop("document", UNSET)
        document: Union[Unset, JobsSinglePostResponseDataRelationshipsDocument]
        if isinstance(_document, Unset):
            document = UNSET
        else:
            document = (
                JobsSinglePostResponseDataRelationshipsDocument.from_dict(
                    _document
                )
            )

        _documents = d.pop("documents", UNSET)
        documents: Union[
            Unset, JobsSinglePostResponseDataRelationshipsDocuments
        ]
        if isinstance(_documents, Unset):
            documents = UNSET
        else:
            documents = (
                JobsSinglePostResponseDataRelationshipsDocuments.from_dict(
                    _documents
                )
            )

        _project = d.pop("project", UNSET)
        project: Union[Unset, JobsSinglePostResponseDataRelationshipsProject]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = JobsSinglePostResponseDataRelationshipsProject.from_dict(
                _project
            )

        jobs_single_post_response_data_relationships_obj = cls(
            document=document,
            documents=documents,
            project=project,
        )

        jobs_single_post_response_data_relationships_obj.additional_properties = (
            d
        )
        return jobs_single_post_response_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class JobsSingleGetResponseDataAttributesStatusType(str, Enum):
    CANCELLED = "CANCELLED"
    FAILED = "FAILED"
    OK = "OK"
    UNKNOWN = "UNKNOWN"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SparseFields")


@_attrs_define
class SparseFields:
    """
    Attributes:
        categories (Union[Unset, str]): Requested fields Example: @all.
        document_attachments (Union[Unset, str]): Requested fields Example: @all.
        document_comments (Union[Unset, str]): Requested fields Example: @all.
        document_parts (Union[Unset, str]): Requested fields Example: @all.
        documents (Union[Unset, str]): Requested fields Example: @all.
        enumerations (Union[Unset, str]): Requested fields Example: @all.
        externallylinkedworkitems (Union[Unset, str]): Requested fields Example: @all.
        featureselections (Union[Unset, str]): Requested fields Example: @all.
        globalroles (Union[Unset, str]): Requested fields Example: @all.
        icons (Union[Unset, str]): Requested fields Example: @all.
        jobs (Union[Unset, str]): Requested fields Example: @all.
        linkedoslcresources (Union[Unset, str]): Requested fields Example: @all.
        linkedworkitems (Union[Unset, str]): Requested fields Example: @all.
        page_attachments (Union[Unset, str]): Requested fields Example: @all.
        pages (Union[Unset, str]): Requested fields Example: @all.
        plans (Union[Unset, str]): Requested fields Example: @all.
        projectroles (Union[Unset, str]): Requested fields Example: @all.
        projects (Union[Unset, str]): Requested fields Example: @all.
        projecttemplates (Union[Unset, str]): Requested fields Example: @all.
        revisions (Union[Unset, str]): Requested fields Example: @all.
        testparameter_definitions (Union[Unset, str]): Requested fields Example: @all.
        testparameters (Union[Unset, str]): Requested fields Example: @all.
        testrecord_attachments (Union[Unset, str]): Requested fields Example: @all.
        testrecords (Union[Unset, str]): Requested fields Example: @all.
        testrun_attachments (Union[Unset, str]): Requested fields Example: @all.
        testrun_comments (Union[Unset, str]): Requested fields Example: @all.
        testruns (Union[Unset, str]): Requested fields Example: @all.
        teststep_results (Union[Unset, str]): Requested fields Example: @all.
        teststepresult_attachments (Union[Unset, str]): Requested fields Example: @all.
        teststeps (Union[Unset, str]): Requested fields Example: @all.
        usergroups (Union[Unset, str]): Requested fields Example: @all.
        users (Union[Unset, str]): Requested fields Example: @all.
        workitem_approvals (Union[Unset, str]): Requested fields Example: @all.
        workitem_attachments (Union[Unset, str]): Requested fields Example: @all.
        workitem_comments (Union[Unset, str]): Requested fields Example: @all.
        workitems (Union[Unset, str]): Requested fields Example: @all.
        workrecords (Union[Unset, str]): Requested fields Example: @all.
    """

    categories: Union[Unset, str] = UNSET
    document_attachments: Union[Unset, str] = UNSET
    document_comments: Union[Unset, str] = UNSET
    document_parts: Union[Unset, str] = UNSET
    documents: Union[Unset, str] = UNSET
    enumerations: Union[Unset, str] = UNSET
    externallylinkedworkitems: Union[Unset, str] = UNSET
    featureselections: Union[Unset, str] = UNSET
    globalroles: Union[Unset, str] = UNSET
    icons: Union[Unset, str] = UNSET
    jobs: Union[Unset, str] = UNSET
    linkedoslcresources: Union[Unset, str] = UNSET
    linkedworkitems: Union[Unset, str] = UNSET
    page_attachments: Union[Unset, str] = UNSET
    pages: Union[Unset, str] = UNSET
    plans: Union[Unset, str] = UNSET
    projectroles: Union[Unset, str] = UNSET
    projects: Union[Unset, str] = UNSET
    projecttemplates: Union[Unset, str] = UNSET
    revisions: Union[Unset, str] = UNSET
    testparameter_definitions: Union[Unset, str] = UNSET
    testparameters: Union[Unset, str] = UNSET
    testrecord_attachments: Union[Unset, str] = UNSET
    testrecords: Union[Unset, str] = UNSET
    testrun_attachments: Union[Unset, str] = UNSET
    testrun_comments: Union[Unset, str] = UNSET
    testruns: Union[Unset, str] = UNSET
    teststep_results: Union[Unset, str] = UNSET
    teststepresult_attachments: Union[Unset, str] = UNSET
    teststeps: Union[Unset, str] = UNSET
    usergroups: Union[Unset, str] = UNSET
    users: Union[Unset, str] = UNSET
    workitem_approvals: Union[Unset, str] = UNSET
    workitem_attachments: Union[Unset, str] = UNSET
    workitem_comments: Union[Unset, str] = UNSET
    workitems: Union[Unset, str] = UNSET
    workrecords: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        categories = self.categories

        document_attachments = self.document_attachments

        document_comments = self.document_comments

        document_parts = self.document_parts

        documents = self.documents

        enumerations = self.enumerations

        externallylinkedworkitems = self.externallylinkedworkitems

        featureselections = self.featureselections

        globalroles = self.globalroles

        icons = self.icons

        jobs = self.jobs

        linkedoslcresources = self.linkedoslcresources

        linkedworkitems = self.linkedworkitems

        page_attachments = self.page_attachments

        pages = self.pages

        plans = self.plans

        projectroles = self.projectroles

        projects = self.projects

        projecttemplates = self.projecttemplates

        revisions = self.revisions

        testparameter_definitions = self.testparameter_definitions

        testparameters = self.testparameters

        testrecord_attachments = self.testrecord_attachments

        testrecords = self.testrecords

        testrun_attachments = self.testrun_attachments

        testrun_comments = self.testrun_comments

        testruns = self.testruns

        teststep_results = self.teststep_results

        teststepresult_attachments = self.teststepresult_attachments

        teststeps = self.teststeps

        usergroups = self.usergroups

        users = self.users

        workitem_approvals = self.workitem_approvals

        workitem_attachments = self.workitem_attachments

        workitem_comments = self.workitem_comments

        workitems = self.workitems

        workrecords = self.workrecords

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if categories is not UNSET:
            field_dict["categories"] = categories
        if document_attachments is not UNSET:
            field_dict["document_attachments"] = document_attachments
        if document_comments is not UNSET:
            field_dict["document_comments"] = document_comments
        if document_parts is not UNSET:
            field_dict["document_parts"] = document_parts
        if documents is not UNSET:
            field_dict["documents"] = documents
        if enumerations is not UNSET:
            field_dict["enumerations"] = enumerations
        if externallylinkedworkitems is not UNSET:
            field_dict["externallylinkedworkitems"] = externallylinkedworkitems
        if featureselections is not UNSET:
            field_dict["featureselections"] = featureselections
        if globalroles is not UNSET:
            field_dict["globalroles"] = globalroles
        if icons is not UNSET:
            field_dict["icons"] = icons
        if jobs is not UNSET:
            field_dict["jobs"] = jobs
        if linkedoslcresources is not UNSET:
            field_dict["linkedoslcresources"] = linkedoslcresources
        if linkedworkitems is not UNSET:
            field_dict["linkedworkitems"] = linkedworkitems
        if page_attachments is not UNSET:
            field_dict["page_attachments"] = page_attachments
        if pages is not UNSET:
            field_dict["pages"] = pages
        if plans is not UNSET:
            field_dict["plans"] = plans
        if projectroles is not UNSET:
            field_dict["projectroles"] = projectroles
        if projects is not UNSET:
            field_dict["projects"] = projects
        if projecttemplates is not UNSET:
            field_dict["projecttemplates"] = projecttemplates
        if revisions is not UNSET:
            field_dict["revisions"] = revisions
        if testparameter_definitions is not UNSET:
            field_dict["testparameter_definitions"] = testparameter_definitions
        if testparameters is not UNSET:
            field_dict["testparameters"] = testparameters
        if testrecord_attachments is not UNSET:
            field_dict["testrecord_attachments"] = testrecord_attachments
        if testrecords is not UNSET:
            field_dict["testrecords"] = testrecords
        if testrun_attachments is not UNSET:
            field_dict["testrun_attachments"] = testrun_attachments
        if testrun_comments is not UNSET:
            field_dict["testrun_comments"] = testrun_comments
        if testruns is not UNSET:
            field_dict["testruns"] = testruns
        if teststep_results is not UNSET:
            field_dict["teststep_results"] = teststep_results
        if teststepresult_attachments is not UNSET:
            field_dict["teststepresult_attachments"] = (
                teststepresult_attachments
            )
        if teststeps is not UNSET:
            field_dict["teststeps"] = teststeps
        if usergroups is not UNSET:
            field_dict["usergroups"] = usergroups
        if users is not UNSET:
            field_dict["users"] = users
        if workitem_approvals is not UNSET:
            field_dict["workitem_approvals"] = workitem_approvals
        if workitem_attachments is not UNSET:
            field_dict["workitem_attachments"] = workitem_attachments
        if workitem_comments is not UNSET:
            field_dict["workitem_comments"] = workitem_comments
        if workitems is not UNSET:
            field_dict["workitems"] = workitems
        if workrecords is not UNSET:
            field_dict["workrecords"] = workrecords

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        categories = d.pop("categories", UNSET)

        document_attachments = d.pop("document_attachments", UNSET)

        document_comments = d.pop("document_comments", UNSET)

        document_parts = d.pop("document_parts", UNSET)

        documents = d.pop("documents", UNSET)

        enumerations = d.pop("enumerations", UNSET)

        externallylinkedworkitems = d.pop("externallylinkedworkitems", UNSET)

        featureselections = d.pop("featureselections", UNSET)

        globalroles = d.pop("globalroles", UNSET)

        icons = d.pop("icons", UNSET)

        jobs = d.pop("jobs", UNSET)

        linkedoslcresources = d.pop("linkedoslcresources", UNSET)

        linkedworkitems = d.pop("linkedworkitems", UNSET)

        page_attachments = d.pop("page_attachments", UNSET)

        pages = d.pop("pages", UNSET)

        plans = d.pop("plans", UNSET)

        projectroles = d.pop("projectroles", UNSET)

        projects = d.pop("projects", UNSET)

        projecttemplates = d.pop("projecttemplates", UNSET)

        revisions = d.pop("revisions", UNSET)

        testparameter_definitions = d.pop("testparameter_definitions", UNSET)

        testparameters = d.pop("testparameters", UNSET)

        testrecord_attachments = d.pop("testrecord_attachments", UNSET)

        testrecords = d.pop("testrecords", UNSET)

        testrun_attachments = d.pop("testrun_attachments", UNSET)

        testrun_comments = d.pop("testrun_comments", UNSET)

        testruns = d.pop("testruns", UNSET)

        teststep_results = d.pop("teststep_results", UNSET)

        teststepresult_attachments = d.pop("teststepresult_attachments", UNSET)

        teststeps = d.pop("teststeps", UNSET)

        usergroups = d.pop("usergroups", UNSET)

        users = d.pop("users", UNSET)

        workitem_approvals = d.pop("workitem_approvals", UNSET)

        workitem_attachments = d.pop("workitem_attachments", UNSET)

        workitem_comments = d.pop("workitem_comments", UNSET)

        workitems = d.pop("workitems", UNSET)

        workrecords = d.pop("workrecords", UNSET)

        sparse_fields_obj = cls(
            categories=categories,
            document_attachments=document_attachments,
            document_comments=document_comments,
            document_parts=document_parts,
            documents=documents,
            enumerations=enumerations,
            externallylinkedworkitems=externallylinkedworkitems,
            featureselections=featureselections,
            globalroles=globalroles,
            icons=icons,
            jobs=jobs,
            linkedoslcresources=linkedoslcresources,
            linkedworkitems=linkedworkitems,
            page_attachments=page_attachments,
            pages=pages,
            plans=plans,
            projectroles=projectroles,
            projects=projects,
            projecttemplates=projecttemplates,
            revisions=revisions,
            testparameter_definitions=testparameter_definitions,
            testparameters=testparameters,
            testrecord_attachments=testrecord_attachments,
            testrecords=testrecords,
            testrun_attachments=testrun_attachments,
            testrun_comments=testrun_comments,
            testruns=testruns,
            teststep_results=teststep_results,
            teststepresult_attachments=teststepresult_attachments,
            teststeps=teststeps,
            usergroups=usergroups,
            users=users,
            workitem_approvals=workitem_approvals,
            workitem_attachments=workitem_attachments,
            workitem_comments=workitem_comments,
            workitems=workitems,
            workrecords=workrecords,
        )

        sparse_fields_obj.additional_properties = d
        return sparse_fields_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar(
    "T", bound="EnumerationsSinglePatchRequestDataAttributesOptionsItem"
)


@_attrs_define
class EnumerationsSinglePatchRequestDataAttributesOptionsItem:
    """
    Attributes:
        color (Union[Unset, str]):  Example: #F9FF4D.
        column_width (Union[Unset, str]):  Example: 90%.
        create_defect (Union[Unset, bool]):  Example: True.
        default (Union[Unset, bool]):  Example: True.
        description (Union[Unset, str]):  Example: Description.
        hidden (Union[Unset, bool]):
        icon_url (Union[Unset, str]):  Example: /polarion/icons/default/enums/status_open.gif.
        id (Union[Unset, str]):  Example: open.
        min_value (Union[Unset, float]):  Example: 30.0.
        name (Union[Unset, str]):  Example: Open.
        opposite_name (Union[Unset, str]):  Example: Opposite Name.
        parent (Union[Unset, bool]):  Example: True.
        requires_signature_for_test_case_execution (Union[Unset, bool]):  Example: True.
        template_work_item (Union[Unset, str]):  Example: exampleTemplate.
        terminal (Union[Unset, bool]):  Example: True.
    """

    color: Union[Unset, str] = UNSET
    column_width: Union[Unset, str] = UNSET
    create_defect: Union[Unset, bool] = UNSET
    default: Union[Unset, bool] = UNSET
    description: Union[Unset, str] = UNSET
    hidden: Union[Unset, bool] = UNSET
    icon_url: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    min_value: Union[Unset, float] = UNSET
    name: Union[Unset, str] = UNSET
    opposite_name: Union[Unset, str] = UNSET
    parent: Union[Unset, bool] = UNSET
    requires_signature_for_test_case_execution: Union[Unset, bool] = UNSET
    template_work_item: Union[Unset, str] = UNSET
    terminal: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        color = self.color

        column_width = self.column_width

        create_defect = self.create_defect

        default = self.default

        description = self.description

        hidden = self.hidden

        icon_url = self.icon_url

        id = self.id

        min_value = self.min_value

        name = self.name

        opposite_name = self.opposite_name

        parent = self.parent

        requires_signature_for_test_case_execution = (
            self.requires_signature_for_test_case_execution
        )

        template_work_item = self.template_work_item

        terminal = self.terminal

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if color is not UNSET:
            field_dict["color"] = color
        if column_width is not UNSET:
            field_dict["columnWidth"] = column_width
        if create_defect is not UNSET:
            field_dict["createDefect"] = create_defect
        if default is not UNSET:
            field_dict["default"] = default
        if description is not UNSET:
            field_dict["description"] = description
        if hidden is not UNSET:
            field_dict["hidden"] = hidden
        if icon_url is not UNSET:
            field_dict["iconURL"] = icon_url
        if id is not UNSET:
            field_dict["id"] = id
        if min_value is not UNSET:
            field_dict["minValue"] = min_value
        if name is not UNSET:
            field_dict["name"] = name
        if opposite_name is not UNSET:
            field_dict["oppositeName"] = opposite_name
        if parent is not UNSET:
            field_dict["parent"] = parent
        if requires_signature_for_test_case_execution is not UNSET:
            field_dict["requiresSignatureForTestCaseExecution"] = (
                requires_signature_for_test_case_execution
            )
        if template_work_item is not UNSET:
            field_dict["templateWorkItem"] = template_work_item
        if terminal is not UNSET:
            field_dict["terminal"] = terminal

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        color = d.pop("color", UNSET)

        column_width = d.pop("columnWidth", UNSET)

        create_defect = d.pop("createDefect", UNSET)

        default = d.pop("default", UNSET)

        description = d.pop("description", UNSET)

        hidden = d.pop("hidden", UNSET)

        icon_url = d.pop("iconURL", UNSET)

        id = d.pop("id", UNSET)

        min_value = d.pop("minValue", UNSET)

        name = d.pop("name", UNSET)

        opposite_name = d.pop("oppositeName", UNSET)

        parent = d.pop("parent", UNSET)

        requires_signature_for_test_case_execution = d.pop(
            "requiresSignatureForTestCaseExecution", UNSET
        )

        template_work_item = d.pop("templateWorkItem", UNSET)

        terminal = d.pop("terminal", UNSET)

        enumerations_single_patch_request_data_attributes_options_item_obj = cls(
            color=color,
            column_width=column_width,
            create_defect=create_defect,
            default=default,
            description=description,
            hidden=hidden,
            icon_url=icon_url,
            id=id,
            min_value=min_value,
            name=name,
            opposite_name=opposite_name,
            parent=parent,
            requires_signature_for_test_case_execution=requires_signature_for_test_case_execution,
            template_work_item=template_work_item,
            terminal=terminal,
        )

        enumerations_single_patch_request_data_attributes_options_item_obj.additional_properties = (
            d
        )
        return (
            enumerations_single_patch_request_data_attributes_options_item_obj
        )

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Implementation of the documents client."""
import itertools
import typing as t
import urllib.parse

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.documents import (
    get_document,
    patch_document,
    post_documents,
)

from . import base_classes as bc


class Documents(
    bc.SingleUpdatableItemsMixin[dm.Document],
    bc.UpdatableItemsClient[dm.Document],
):
    """A client to work with documents in Polarion."""

    def get(
        self,
        space_id: str,
        document_name: str,
        fields: dict[str, str] | None = None,
        include: str | None | oa_types.Unset = None,
        revision: str | None | oa_types.Unset = None,
    ) -> dm.Document | None:
        """Return the document with the given document_name and space_id."""
        if include is None:
            include = oa_types.UNSET

        if revision is None:
            revision = oa_types.UNSET

        if " " in space_id or " " in document_name:
            space_id = urllib.parse.quote(
                space_id, safe="/", encoding=None, errors=None
            )
            document_name = urllib.parse.quote(
                document_name, safe="/", encoding=None, errors=None
            )
        if fields is None:
            fields = self._client.default_fields.documents

        sparse_fields = self._build_sparse_fields(fields)
        response = get_document.sync_detailed(
            self._project_id,
            space_id,
            document_name,
            client=self._client.client,
            fields=sparse_fields,
            include=include,
            revision=revision,
        )

        self._raise_on_error(response)

        document_response = response.parsed

        if isinstance(
            document_response, api_models.DocumentsSingleGetResponse
        ) and (data := document_response.data):
            if not getattr(data.meta, "errors", []):
                assert (attributes := data.attributes)
                assert isinstance(data.id, str)
                home_page_content = self._handle_text_content(
                    attributes.home_page_content
                )

                rendering_layouts = None
                if attributes.rendering_layouts:
                    rendering_layouts = [
                        dm.RenderingLayout(
                            self.unset_to_none(layout.label),
                            self.unset_to_none(layout.layouter),
                            (
                                [p.to_dict() for p in layout.properties]
                                if layout.properties
                                else None
                            ),
                            self.unset_to_none(layout.type),
                        )
                        for layout in attributes.rendering_layouts
                    ]

                return dm.Document(
                    id=data.id,
                    module_folder=self.unset_to_none(attributes.module_folder),
                    module_name=self.unset_to_none(attributes.module_name),
                    type=self.unset_to_none(attributes.type),
                    status=self.unset_to_none(attributes.status),
                    home_page_content=home_page_content,
                    title=self.unset_to_none(attributes.title),
                    rendering_layouts=rendering_layouts,
                    outline_numbering=self.unset_to_none(
                        attributes.uses_outline_numbering
                    ),
                    outline_numbering_prefix=(
                        self.unset_to_none(attributes.outline_numbering.prefix)
                        if attributes.outline_numbering
                        else None
                    ),
                )

        return None

    def _split_into_batches(
        self, items: list[dm.Document]
    ) -> t.Generator[list[dm.Document], None, None]:
        for _, group in itertools.groupby(items, lambda x: x.module_folder):
            yield from super()._split_into_batches(list(group))

    def _update(self, to_update: dm.Document | list[dm.Document]):
        assert not isinstance(to_update, list), "Expected only one item"
        assert to_update.module_folder is not None, "module folder must be set"
        assert to_update.module_name is not None, "module name must be set"
        # pylint: disable=line-too-long
        req = api_models.DocumentsSinglePatchRequest(
            data=api_models.DocumentsSinglePatchRequestData(
                api_models.DocumentsSinglePatchRequestDataType.DOCUMENTS,
                id=f"{self._project_id}/{to_update.module_folder}/{to_update.module_name}",
                attributes=api_models.DocumentsSinglePatchRequestDataAttributes(
                    home_page_content=(
                        api_models.DocumentsSinglePatchRequestDataAttributesHomePageContent(
                            type=api_models.DocumentsSinglePatchRequestDataAttributesHomePageContentType(
                                to_update.home_page_content.type
                            ),
                            value=to_update.home_page_content.value
                            or oa_types.UNSET,
                        )
                        if to_update.home_page_content
                        else oa_types.UNSET
                    ),
                    status=to_update.status or oa_types.UNSET,
                    title=to_update.title or oa_types.UNSET,
                    type=to_update.type or oa_types.UNSET,
                    rendering_layouts=(
                        [
                            api_models.DocumentsSinglePatchRequestDataAttributesRenderingLayoutsItem(
                                label=layout.label or oa_types.UNSET,
                                layouter=(
                                    layout.layouter.value
                                    if layout.layouter is not None
                                    else oa_types.UNSET
                                ),
                                type=layout.type or oa_types.UNSET,
                                properties=(
                                    [
                                        api_models.DocumentsSinglePatchRequestDataAttributesRenderingLayoutsItemPropertiesItem.from_dict(
                                            p
                                        )
                                        for p in layout.properties.serialize()
                                    ]
                                    if layout.properties
                                    else oa_types.UNSET
                                ),
                            )
                            for layout in to_update.rendering_layouts
                        ]
                        if to_update.rendering_layouts
                        else oa_types.UNSET
                    ),
                    uses_outline_numbering=to_update.outline_numbering
                    or oa_types.UNSET,
                    outline_numbering=(
                        api_models.DocumentsSinglePatchRequestDataAttributesOutlineNumbering(
                            prefix=to_update.outline_numbering_prefix
                        )
                        if to_update.outline_numbering_prefix
                        else oa_types.UNSET
                    ),
                ),
            )
        )
        # pylint: enable=line-too-long

        res = patch_document.sync_detailed(
            project_id=self._project_id,
            space_id=to_update.module_folder,
            document_name=to_update.module_name,
            client=self._client.client,
            body=req,
        )

        self._raise_on_error(res)

    def get_multi(
        self, *args, page_size=100, page_number=1, **kwargs
    ) -> tuple[list[dm.Document], bool]:
        """Return a list of documents - Not implemented yet."""
        raise NotImplementedError

    def _create(self, items: list[dm.Document]):
        # due to grouping in _split_into_batches all module folders are equal
        assert items[0].module_folder is not None, "module folder must be set"

        req = api_models.DocumentsListPostRequest(
            # pylint: disable=line-too-long
            data=[
                api_models.DocumentsListPostRequestDataItem(
                    type=api_models.DocumentsListPostRequestDataItemType.DOCUMENTS,
                    attributes=api_models.DocumentsListPostRequestDataItemAttributes(
                        home_page_content=(
                            api_models.DocumentsListPostRequestDataItemAttributesHomePageContent(
                                type=api_models.DocumentsListPostRequestDataItemAttributesHomePageContentType(
                                    document.home_page_content.type
                                ),
                                value=document.home_page_content.value
                                or oa_types.UNSET,
                            )
                            if document.home_page_content
                            else oa_types.UNSET
                        ),
                        module_name=document.module_name or oa_types.UNSET,
                        status=document.status or oa_types.UNSET,
                        title=document.title or oa_types.UNSET,
                        type=document.type or oa_types.UNSET,
                        rendering_layouts=(
                            [
                                api_models.DocumentsListPostRequestDataItemAttributesRenderingLayoutsItem(
                                    label=layout.label or oa_types.UNSET,
                                    layouter=(
                                        layout.layouter.value
                                        if layout.layouter is not None
                                        else oa_types.UNSET
                                    ),
                                    type=layout.type or oa_types.UNSET,
                                    properties=(
                                        [
                                            api_models.DocumentsListPostRequestDataItemAttributesRenderingLayoutsItemPropertiesItem.from_dict(
                                                p
                                            )
                                            for p in layout.properties.serialize()
                                        ]
                                        if layout.properties
                                        else oa_types.UNSET
                                    ),
                                )
                                for layout in document.rendering_layouts
                            ]
                            if document.rendering_layouts
                            else oa_types.UNSET
                        ),
                        uses_outline_numbering=document.outline_numbering
                        or oa_types.UNSET,
                        outline_numbering=(
                            api_models.DocumentsListPostRequestDataItemAttributesOutlineNumbering(
                                prefix=document.outline_numbering_prefix
                            )
                            if document.outline_numbering_prefix
                            else oa_types.UNSET
                        ),
                    ),
                )
                for document in items
            ]
            # pylint: enable=line-too-long
        )

        res = post_documents.sync_detailed(
            self._project_id,
            items[0].module_folder,
            client=self._client.client,
            body=req,
        )

        self._raise_on_error(res)

    def _delete(self, items: list[dm.Document]):
        raise NotImplementedError



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.testrecords_single_get_response_data_relationships_defect_data_type import (
    TestrecordsSingleGetResponseDataRelationshipsDefectDataType,
)
from ..types import UNSET, Unset

T = TypeVar(
    "T", bound="TestrecordsSingleGetResponseDataRelationshipsDefectData"
)


@_attrs_define
class TestrecordsSingleGetResponseDataRelationshipsDefectData:
    """
    Attributes:
        id (Union[Unset, str]):  Example: MyProjectId/MyWorkItemId.
        revision (Union[Unset, str]):  Example: 1234.
        type (Union[Unset, TestrecordsSingleGetResponseDataRelationshipsDefectDataType]):
    """

    id: Union[Unset, str] = UNSET
    revision: Union[Unset, str] = UNSET
    type: Union[
        Unset, TestrecordsSingleGetResponseDataRelationshipsDefectDataType
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        revision = self.revision

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if revision is not UNSET:
            field_dict["revision"] = revision
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        revision = d.pop("revision", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[
            Unset, TestrecordsSingleGetResponseDataRelationshipsDefectDataType
        ]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = TestrecordsSingleGetResponseDataRelationshipsDefectDataType(
                _type
            )

        testrecords_single_get_response_data_relationships_defect_data_obj = (
            cls(
                id=id,
                revision=revision,
                type=type,
            )
        )

        testrecords_single_get_response_data_relationships_defect_data_obj.additional_properties = (
            d
        )
        return (
            testrecords_single_get_response_data_relationships_defect_data_obj
        )

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pages_single_patch_request_data import (
        PagesSinglePatchRequestData,
    )


T = TypeVar("T", bound="PagesSinglePatchRequest")


@_attrs_define
class PagesSinglePatchRequest:
    """
    Attributes:
        data (Union[Unset, PagesSinglePatchRequestData]):
    """

    data: Union[Unset, "PagesSinglePatchRequestData"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.pages_single_patch_request_data import (
            PagesSinglePatchRequestData,
        )

        d = src_dict.copy()
        _data = d.pop("data", UNSET)
        data: Union[Unset, PagesSinglePatchRequestData]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = PagesSinglePatchRequestData.from_dict(_data)

        pages_single_patch_request_obj = cls(
            data=data,
        )

        pages_single_patch_request_obj.additional_properties = d
        return pages_single_patch_request_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
